@@ -26,22 +26,22 @@ class QStringPrinter:
         self.val = val
 
     def to_string(self):
-        #ret = ""
-        #i = 0
         size = self.val['d']['size']
-        #while i < size:
-        #    char = self.val['d']['data'][i]
-        #    if (char > 127):
-        #        ret += "\\u%x" % int(char)
-        #    else:
-        #        ret += chr(char)
-        #    i = i + 1
-        #return ret
         ret = ""
+        qt5 = 0
+        try:
+            # Qt4 has d->data, Qt5 doesn't.
+            self.val['d']['data']
+        except Exception:
+            qt5 = 1
+
         # The QString object might be not yet initialized. In this case size is a bogus value
         # and the following 2 lines might throw memory access error. Hence the try/catch.
         try:
-            dataAsCharPointer = self.val['d']['data'].cast(gdb.lookup_type("char").pointer())
+            if qt5:
+                dataAsCharPointer = (self.val['d'] + 1).cast(gdb.lookup_type("char").pointer())
+            else:
+                dataAsCharPointer = self.val['d']['data'].cast(gdb.lookup_type("char").pointer())
             ret = dataAsCharPointer.string(encoding = 'UTF-16', length = size * 2)
         except Exception:
             # swallow the exception and return empty string
@@ -493,7 +493,7 @@ class QUrlPrinter:
     def to_string(self):
         try:
             return self.val['d']['encodedOriginal']
-        except RuntimeError, error:
+        except RuntimeError:
             #if no debug information is avaliable for Qt, try guessing the correct address for encodedOriginal
             #problem with this is that if QUrlPrivate members get changed, this fails
             offset = gdb.lookup_type('int').sizeof
@@ -557,6 +557,21 @@ class QCharPrinter:
     def display_hint (self):
         return 'string'
 
+class QUuidPrinter:
+
+    def __init__(self, val):
+        self.val = val
+
+    def to_string(self):
+        return "QUuid({%x-%x-%x-%x%x-%x%x%x%x%x%x})" % (self.val['data1'], self.val['data2'], self.val['data3'],
+                                            self.val['data4'][0], self.val['data4'][1],
+                                            self.val['data4'][2], self.val['data4'][3],
+                                            self.val['data4'][4], self.val['data4'][5],
+                                            self.val['data4'][6], self.val['data4'][7])
+
+    def display_hint (self):
+        return 'string'
+
 def register_qt4_printers (obj):
     if obj == None:
         obj = gdb
@@ -610,6 +625,7 @@ def build_dictionary ():
     pretty_printers_dict[re.compile('^QUrl$')] = lambda val: QUrlPrinter(val)
     pretty_printers_dict[re.compile('^QSet<.*>$')] = lambda val: QSetPrinter(val)
     pretty_printers_dict[re.compile('^QChar$')] = lambda val: QCharPrinter(val)
+    pretty_printers_dict[re.compile('^QUuid')] = lambda val: QUuidPrinter(val)
 
 
 pretty_printers_dict = {}
