@@ -46,6 +46,8 @@
 #include "gdbcommand.h"
 #include "debugsession.h"
 #include "gdbframestackmodel.h"
+#include <mi/milexer.h>
+#include <mi/miparser.h>
 
 using KDevelop::AutoTestShell;
 
@@ -121,25 +123,31 @@ class TestFrameStackModel : public KDevelop::GdbFrameStackModel
 {
 public:
     
-    TestFrameStackModel(DebugSession* session) 
-    : GdbFrameStackModel(session), fetchFramesCalled(0) {}
-    
+    TestFrameStackModel(DebugSession* session)
+        : GdbFrameStackModel(session), fetchFramesCalled(0), fetchThreadsCalled(0) {}
+
     int fetchFramesCalled;
+    int fetchThreadsCalled;
     virtual void fetchFrames(int threadNumber, int from, int to)
     {
         fetchFramesCalled++;
         GdbFrameStackModel::fetchFrames(threadNumber, from, to);
     }
+
+    virtual void fetchThreads()
+    {
+        fetchThreadsCalled++;
+        GdbFrameStackModel::fetchThreads();
+    }
 };
 
 class TestDebugSession : public DebugSession
 {
     Q_OBJECT
 public:
-    TestDebugSession() : DebugSession(), m_line(0)
+    TestDebugSession() : DebugSession()
     {
         qRegisterMetaType<KUrl>("KUrl");
-        Q_ASSERT(connect(this, SIGNAL(showStepInSource(KUrl,int,QString)), SLOT(slotShowStepInSource(KUrl,int))));
         
         KDevelop::ICore::self()->debugController()->addSession(this);
     }
@@ -149,20 +157,11 @@ public:
         return new TestFrameStackModel(this);
     }
         
-    KUrl url() { return m_url; }
-    int line() { return m_line; }
+    KUrl url() { return currentUrl(); }
+    int line() { return currentLine(); }
     TestFrameStackModel *frameStackModel() const 
     { return static_cast<TestFrameStackModel*>(DebugSession::frameStackModel()); }
 
-private slots:
-    void slotShowStepInSource(const KUrl &url, int line)
-    {
-        m_url = url;
-        m_line = line;
-    }
-private:
-    KUrl m_url;
-    int m_line;
 
 };
 
@@ -810,7 +809,7 @@ void GdbTest::testAttach()
     KProcess debugeeProcess;
     debugeeProcess << "nice" << findExecutable("debugeeslow").toLocalFile();
     debugeeProcess.start();
-    Q_ASSERT(debugeeProcess.waitForStarted());
+    QVERIFY(debugeeProcess.waitForStarted());
     QTest::qWait(100);
 
     TestDebugSession *session = new TestDebugSession;
@@ -839,9 +838,8 @@ void GdbTest::testCoreFile()
     debugeeProcess.start();
     debugeeProcess.waitForFinished();
     kDebug() << debugeeProcess.readAll();
-    QFile f2("core");
-    if (!f2.exists()) {
-        QFAIL("no core dump found");
+    if (!QFile::exists("core")) {
+        QSKIP("no core dump found, check your system configuration (see /proc/sys/kernel/core_pattern).", SkipSingle);
     }
 
     TestDebugSession *session = new TestDebugSession;
@@ -1504,8 +1502,6 @@ void GdbTest::testBreakpointWithSpaceInPath()
 
     session->startProgram(&cfg);
     WAIT_FOR_STATE_FAIL(session, DebugSession::PausedState);
-    //see upstream bugreport: http://sourceware.org/bugzilla/show_bug.cgi?id=13798
-    QEXPECT_FAIL("", "this does not work in gdb 7.4", Abort);
     QCOMPARE(session->line(), 20);
     session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
@@ -1532,10 +1528,159 @@ void GdbTest::testBreakpointDisabledOnStart()
     QCOMPARE(session->line(), 31);
     session->run();
     WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
+
+void GdbTest::testCatchpoint()
+{
+    TestDebugSession *session = new TestDebugSession;
+    session->variableController()->setAutoUpdate(KDevelop::IVariableController::UpdateLocals);
+
+    TestLaunchConfiguration cfg(findExecutable("debugeeexception"));
+
+    breakpoints()->addCodeBreakpoint(findSourceFile("debugeeexception.cpp"), 29);
+
+    session->startProgram(&cfg);
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    QTest::qWait(1000);
+    TestFrameStackModel* fsModel = session->frameStackModel();
+    QCOMPARE(fsModel->currentFrame(), 0);
+    QCOMPARE(session->line(), 29);
+
+    QModelIndex i = variableCollection()->index(1, 0);
+    COMPARE_DATA(i, "Locals");
+    qDebug() << variableCollection()->index(0, 1, i);
+    qDebug() << variableCollection()->index(1, 1, i);
+    QCOMPARE(variableCollection()->rowCount(i), 2);
+
+    session->addCommand(new GDBCommand(GDBMI::NonMI, "catch throw"));
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    QTest::qWait(1000);
+    QCOMPARE(fsModel->currentFrame(), 1); //first frame skiped because somewhere inside stdlib
+    QCOMPARE(session->line(), 22);
+
+    qDebug() << variableCollection()->index(0, 1, i);
+    qDebug() << variableCollection()->index(1, 1, i);
+    QCOMPARE(variableCollection()->rowCount(i), 1);
+    COMPARE_DATA(variableCollection()->index(0, 0, i), "i");
+    COMPARE_DATA(variableCollection()->index(0, 1, i), "20");
+
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::EndedState);
 
 }
 
+void GdbTest::testThreadAndFrameInfo()
+{
+    TestDebugSession *session = new TestDebugSession;
+    TestLaunchConfiguration cfg(findExecutable("debugeethreads"));
+    QString fileName = findSourceFile("debugeethreads.cpp");
+    QSignalSpy outputSpy(session, SIGNAL(gdbUserCommandStdout(QString)));
 
+    breakpoints()->addCodeBreakpoint(fileName, 38);
+    QVERIFY(session->startProgram(&cfg));
+    session->frameStackModel()->fetchThreads();
+    session->addCommand(new UserCommand(GDBMI::StackListLocals, QLatin1String("0")));
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    QTest::qWait(1000);
+
+    QVERIFY(outputSpy.count() == 2);
+    QVERIFY(outputSpy.last().at(0).toString().contains(QLatin1String("--thread 1")));
+
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
+
+void GdbTest::parseBug304730()
+{
+    FileSymbol file;
+    file.contents = QByteArray("^done,bkpt={"
+        "number=\"1\",type=\"breakpoint\",disp=\"keep\",enabled=\"y\",addr=\"<MULTIPLE>\",times=\"0\","
+        "original-location=\"/media/portable/Projects/BDSInpainting/PatchMatch/PatchMatch.hpp:231\"},"
+        "{number=\"1.1\",enabled=\"y\",addr=\"0x081d84aa\","
+        "func=\"PatchMatch<itk::Image<itk::CovariantVector<unsigned char, 3u>, 2u> >"
+        "::Propagation<ForwardPropagationNeighbors>(ForwardPropagationNeighbors)\","
+        "file=\"/media/portable/Projects/BDSInpainting/Drivers/../PatchMatch/PatchMatch.hpp\","
+        "fullname=\"/media/portable/Projects/BDSInpainting/PatchMatch/PatchMatch.hpp\",line=\"231\"},"
+        "{number=\"1.2\",enabled=\"y\",addr=\"0x081d8ae2\","
+        "func=\"PatchMatch<itk::Image<itk::CovariantVector<unsigned char, 3u>, 2u> >"
+        "::Propagation<BackwardPropagationNeighbors>(BackwardPropagationNeighbors)\","
+        "file=\"/media/portable/Projects/BDSInpainting/Drivers/../PatchMatch/PatchMatch.hpp\","
+        "fullname=\"/media/portable/Projects/BDSInpainting/PatchMatch/PatchMatch.hpp\",line=\"231\"},"
+        "{number=\"1.3\",enabled=\"y\",addr=\"0x081d911a\","
+        "func=\"PatchMatch<itk::Image<itk::CovariantVector<unsigned char, 3u>, 2u> >"
+        "::Propagation<AllowedPropagationNeighbors>(AllowedPropagationNeighbors)\","
+        "file=\"/media/portable/Projects/BDSInpainting/Drivers/../PatchMatch/PatchMatch.hpp\","
+        "fullname=\"/media/portable/Projects/BDSInpainting/PatchMatch/PatchMatch.hpp\",line=\"231\"}");
+
+    MIParser parser;
+
+    QScopedPointer<GDBMI::Record> record(parser.parse(&file));
+    QVERIFY(!record.isNull());
+}
+
+void GdbTest::testMultipleLocationsBreakpoint()
+{
+    TestDebugSession *session = new TestDebugSession;
+
+    TestLaunchConfiguration cfg(findExecutable("debugeemultilocbreakpoint"));
+
+    breakpoints()->addCodeBreakpoint("aPlusB");
+
+    //TODO check if the additional location breakpoint is added
+
+    session->startProgram(&cfg);
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    QCOMPARE(session->line(), 19);
+
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+    QCOMPARE(session->line(), 23);
+
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
+
+void GdbTest::testBug301287()
+{
+    TestDebugSession *session = new TestDebugSession;
+    session->variableController()->setAutoUpdate(KDevelop::IVariableController::UpdateWatches);
+
+    TestLaunchConfiguration cfg;
+
+    breakpoints()->addCodeBreakpoint(debugeeFileName, 28);
+
+    session->startProgram(&cfg);
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+
+    variableCollection()->watches()->add("argc");
+    QTest::qWait(300);
+
+    QModelIndex i = variableCollection()->index(0, 0);
+    QCOMPARE(variableCollection()->rowCount(i), 1);
+    COMPARE_DATA(variableCollection()->index(0, 0, i), "argc");
+    COMPARE_DATA(variableCollection()->index(0, 1, i), "1");
+
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::EndedState);
+
+    //start second debug session (same cfg)
+    session = new TestDebugSession;
+    session->variableController()->setAutoUpdate(KDevelop::IVariableController::UpdateWatches);
+
+    session->startProgram(&cfg);
+    WAIT_FOR_STATE(session, DebugSession::PausedState);
+
+    QTest::qWait(300);
+
+    i = variableCollection()->index(0, 0);
+    QCOMPARE(variableCollection()->rowCount(i), 1);
+    COMPARE_DATA(variableCollection()->index(0, 0, i), "argc");
+    COMPARE_DATA(variableCollection()->index(0, 1, i), "1");
+
+    session->run();
+    WAIT_FOR_STATE(session, DebugSession::EndedState);
+}
 
 void GdbTest::waitForState(GDBDebugger::DebugSession *session, DebugSession::DebuggerState state,
                             const char *file, int line, bool expectFail)
@@ -1573,3 +1718,4 @@ QTEST_KDEMAIN(GDBDebugger::GdbTest, GUI)
 
 #include "gdbtest.moc"
 #include "moc_gdbtest.cpp"
+
