@@ -59,6 +59,7 @@
 #include <language/duchain/classdeclaration.h>
 #include "qpropertydeclaration.h"
 #include "model.h"
+#include "helpers.h"
 
 // #define ifDebug(x) x
 
@@ -100,7 +101,7 @@ const QSet<QString> BINARY_OPERATORS =
       BINARY_ARITHMETIC_OPERATORS + ARITHMETIC_COMPARISON_OPERATORS;
 //These will be skipped over to find parent contexts
 const QSet<QString> UNARY_OPERATORS = QString("++ -- ! ~ + - & *").split(' ').toSet();
-const QSet<QString> KEYWORD_ACCESS_STRINGS = QString("const_cast< static_cast< dynamic_cast< reinterpret_cast< const typedef public public: protected protected: private private: virtual return else throw emit Q_EMIT case delete delete[] new friend class").split(' ').toSet();
+const QSet<QString> KEYWORD_ACCESS_STRINGS = QString("const_cast< static_cast< dynamic_cast< reinterpret_cast< const typedef public public: protected protected: private private: virtual return else throw emit Q_EMIT case delete delete[] new friend class namespace").split(' ').toSet();
 //When these appear as access strings, only show types
 const QSet<QString> SHOW_TYPES_ACCESS_STRINGS = QString("const_cast< static_cast< dynamic_cast< reinterpret_cast< const typedef public protected private virtual new friend class").split(' ').toSet();
 //A parent context is created for these access strings
@@ -126,26 +127,6 @@ namespace Cpp {
 
 typedef QPair<Declaration*, int> DeclarationDepthPair;
 
-IndexedType switchExpressionType(DUContextPointer caseContext)
-{
-#ifndef TEST_COMPLETION
-  //Test thread has DUChain locked
-  ENSURE_CHAIN_NOT_LOCKED;
-#endif
-  ForegroundLock foregroundLock;
-  LOCKDUCHAIN;
-  if (!caseContext)
-    return IndexedType();
-  DUContext* switchContext = 0;
-  if (caseContext->importedParentContexts().size() == 1)
-    switchContext = caseContext->importedParentContexts().first().context(caseContext->topContext());
-  if (!switchContext)
-    return IndexedType();
-  QString switchExpression = switchContext->createRangeMoving()->text();
-  ExpressionParser expressionParser;
-  return expressionParser.evaluateType(switchExpression.toUtf8(), DUContextPointer(switchContext)).type;
-}
-
 //Search for a copy-constructor within class
 //*DUChain must be locked*
 bool hasCopyConstructor(CppClassType::Ptr classType, TopDUContext* topContext)
@@ -371,6 +352,67 @@ QString getUnaryOperator(const QString &context)
   return unOp;
 }
 
+//Returns the class or struct declaration for the given type
+Declaration* containerDeclForType(AbstractType::Ptr givenType, TopDUContext* top, bool &typeIsPointer)
+{
+  if (PointerType::Ptr ptrType = givenType.cast<PointerType>())
+  {
+    if (!typeIsPointer)
+    {
+      typeIsPointer = true;
+      return containerDeclForType(ptrType->baseType(), top, typeIsPointer);
+    }
+    else
+      return 0; //The given type is a pointer to a pointer, and so cannot be accessed with ->
+  }
+
+  if (TypeAliasType::Ptr typeAliasType = givenType.cast<TypeAliasType>())
+    return containerDeclForType(typeAliasType->type(), top, typeIsPointer);
+
+  if (const IdentifiedType* identifiedType = dynamic_cast<const IdentifiedType*>(givenType.unsafeData()))
+  {
+    if (Declaration *ret = identifiedType->declaration(top))
+    {
+      if (dynamic_cast<ClassDeclaration*>(ret->logicalDeclaration(top)))
+        return ret;
+    }
+  }
+
+  return 0; //Type could not be identified or was not a class declaration
+}
+
+typedef QPair<Declaration*, bool> DeclAccessPair;
+//Look through localDeclarations of "container" for items matching "type"
+QList<DeclAccessPair> containedItemsMatchingType(Declaration *container, const IndexedType& type, TopDUContext *top, bool isPointer)
+{
+  static const Identifier arrowOpIdentifier("operator->");
+  QList<DeclAccessPair> ret;
+  if (!container || !container->internalContext())
+    return ret;
+  Cpp::TypeConversion conv(top);
+  Declaration *arrowOperator = 0;
+  QVector<Declaration*> containedDeclarations = container->internalContext()->localDeclarations(top);
+  foreach(Declaration *decl, containedDeclarations)
+  {
+    if (decl->isTypeAlias() || decl->isForwardDeclaration() || decl->type<EnumerationType>())
+      continue; //Skip declarations that are not accessed via ./->
+
+    if (!isPointer && decl->identifier() == arrowOpIdentifier)
+      arrowOperator = decl;
+
+    AbstractType::Ptr declEffectiveType = Cpp::effectiveType(decl);
+    if (!declEffectiveType.isNull() && conv.implicitConversion(declEffectiveType->indexed(), type))
+      ret << DeclAccessPair(decl, isPointer);
+  }
+  //If we found an "->", try to treat it as a smart pointer
+  if (arrowOperator)
+  {
+    //containerDeclForType will likely change isPointer... good thing we don't use it again
+    ret += containedItemsMatchingType(containerDeclForType(Cpp::effectiveType(arrowOperator), top, isPointer), type, top, true);
+  }
+  return ret;
+}
+
 CodeCompletionContext::
 CodeCompletionContext( KDevelop::DUContextPointer context, const QString& text,
                        const QString& followingText, const KDevelop::CursorInRevision& position,
@@ -850,6 +892,9 @@ CodeCompletionContext::AccessType CodeCompletionContext::findAccessType( const Q
   if( accessStr == "::" )
     return StaticMemberChoose;
 
+  if ( accessStr == "namespace" )
+    return NamespaceAccess;
+
   if ( m_depth > 0 )
   {
     if( accessStr == "(" )
@@ -966,6 +1011,9 @@ CodeCompletionContext::OnlyShow CodeCompletionContext::findOnlyShow(const QStrin
   if ( parentContext() && parentContext()->accessType() == TemplateAccess )
     return ShowTypes;
 
+  if ( parentContext() && parentContext()->accessType() == CaseAccess )
+    return ShowIntegralConstants;
+
   //Only ShowTypes in these DUContexts unless initializing a declaration
   //ie, m_expressionIsTypePrefix == true
   if (m_duContext->type() == DUContext::Class ||
@@ -1045,6 +1093,7 @@ bool CodeCompletionContext::doIncludeCompletion()
   if(!line.startsWith("#"))
     return false;
 
+  m_accessType = IncludeListAccess;
   if(line.count('"') == 2 || line.endsWith('>'))
     return true; //We are behind a complete include-directive
 
@@ -1078,8 +1127,6 @@ bool CodeCompletionContext::doIncludeCompletion()
   m_includeItems = CppUtils::allFilesInIncludePath(KUrl(m_duContext->url().str()), local, prefixPath);
 #endif
 
-  m_accessType = IncludeListAccess;
-
   return true;
 }
 
@@ -1363,20 +1410,27 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::memberAccessCompletionIt
   return items;
 }
 
+AbstractType::Ptr functionReturnType(DUContext* startContext)
+{
+  while(startContext && !startContext->owner())
+    startContext = startContext->parentContext();
+  if(startContext && startContext->owner())
+  {
+    FunctionType::Ptr funType = startContext->owner()->type<FunctionType>();
+    if (funType && funType->returnType())
+      return funType->returnType();
+  }
+  return AbstractType::Ptr();
+}
+
 QList<CompletionTreeItemPointer> CodeCompletionContext::returnAccessCompletionItems()
 {
   QList<CompletionTreeItemPointer> items;
   LOCKDUCHAIN; if (!m_duContext) return items;
 
-  DUContext* functionContext = m_duContext.data();
-  while(functionContext && !functionContext->owner())
-    functionContext = functionContext->parentContext();
-  if(functionContext && functionContext->owner()) {
-    FunctionType::Ptr funType = functionContext->owner()->type<FunctionType>();
-    if(funType && funType->returnType()) {
-        items << CompletionTreeItemPointer( new TypeConversionCompletionItem( "return " + funType->returnType()->toString(), funType->returnType()->indexed(), depth(), KSharedPtr <Cpp::CodeCompletionContext >(this) ) );
-    }
-  }
+  AbstractType::Ptr returnType = functionReturnType(m_duContext.data());
+  if (returnType)
+    items << CompletionTreeItemPointer( new TypeConversionCompletionItem( "return " + returnType->toString(), returnType->indexed(), depth(), KSharedPtr <Cpp::CodeCompletionContext >(this) ) );
   return items;
 }
 
@@ -1384,7 +1438,26 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::caseAccessCompletionItem
 {
   QList<CompletionTreeItemPointer> items;
 
-  IndexedType switchExprType = switchExpressionType(m_duContext);
+  {
+#ifndef TEST_COMPLETION
+    // Test thread has DUChain locked
+    // This is essential here - a ForegroundLock must be acquired _before_ locking DUChain
+    ENSURE_CHAIN_NOT_LOCKED;
+#endif
+
+    ForegroundLock foregroundLock;
+    LOCKDUCHAIN;
+
+    if( m_duContext && m_duContext->importedParentContexts().size() == 1 )
+    {
+      DUContext* switchContext = m_duContext->importedParentContexts().first().context( m_duContext->topContext() );
+      ExpressionParser expressionParser;
+      m_expression = switchContext->createRangeMoving()->text();
+      m_expressionResult = expressionParser.evaluateExpression( m_expression.toUtf8(), DUContextPointer( switchContext ) );
+    }
+  }
+
+  IndexedType switchExprType = m_expressionResult.type;
 
   LOCKDUCHAIN; if (!m_duContext) return items;
 
@@ -1612,6 +1685,54 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::signalSlotAccessCompleti
   return items;
 }
 
+QList<IndexedType> CodeCompletionContext::matchTypes()
+{
+  QSet<KDevelop::IndexedType> ret;
+  if (m_accessType == BinaryOpFunctionCallAccess || m_accessType == FunctionCallAccess )
+  {
+    //MatchTypes for custom operator functions
+    foreach(const Function &func, m_matchingFunctionOverloads)
+    {
+      if (!func.function.isValid() || !func.function.isViable() || !func.function.declaration())
+        continue;
+      FunctionType::Ptr funcType = func.function.declaration()->type<FunctionType>();
+      if(funcType && funcType->indexedArgumentsSize() > (uint)func.matchedArguments)
+        ret << funcType->indexedArguments()[func.matchedArguments];
+    }
+    /* generally matching the other side's type is only useful for "=" or "==" ... Consider:
+     * if (foo && foo == "str" || <complete here>)
+     * The expressionResult preceeding the "||" operator is ["str"] and not [foo == "str"] as it should be
+     * therefore the matchType will be "const char *" and not "boolean" as it should be
+     * If this is fixed, we could use more operators here. See expressionBefore().
+     */
+    if (!m_matchingFunctionOverloads.size() && (m_operator == "=" || m_operator == "=="))
+      ret << m_expressionResult.type;
+  }
+  else if (m_accessType == ReturnAccess)
+  {
+    if (AbstractType::Ptr returnType = functionReturnType(m_duContext.data()))
+      ret << returnType->indexed();
+  }
+  //As of now, no other access types have any matchTypes
+  return ret.toList();
+}
+
+QList<DeclAccessPair> CodeCompletionContext::getLookaheadMatches(Declaration* forDecl, const QList<IndexedType>& matchTypes) const
+{
+  QList<DeclAccessPair> ret;
+  // feature disabled for KDevelop 4.5
+  return ret;
+
+  if (forDecl->isFunctionDeclaration() || forDecl->kind() != Declaration::Instance || !forDecl->abstractType())
+    return ret; //We can only use instances, for now no sub decls of functions either TODO: be nice to get no-arg functions at least
+  bool typeIsPointer = false;
+  Declaration *containerDecl = containerDeclForType(Cpp::effectiveType(forDecl), m_duContext->topContext(), typeIsPointer);
+  foreach(const IndexedType &matchType, matchTypes)
+    ret += containedItemsMatchingType(containerDecl, matchType, m_duContext->topContext(), typeIsPointer);
+  //Could use hideOverloadedDeclarations theoretically here, but it would do very little since we don't have the real declaration depth
+  return ret;
+}
+
 QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompletionItems() {
   QList<CompletionTreeItemPointer> items;
   LOCKDUCHAIN; if (!m_duContext) return items;
@@ -1623,7 +1744,6 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
     if (func->abstractType() && (func->abstractType()->modifiers() & AbstractType::ConstModifier))
       typeIsConst = true;
   }
-
   QList<DeclarationDepthPair> decls = m_duContext->allDeclarations(m_duContext->type() == DUContext::Class ? m_duContext->range().end : m_position, m_duContext->topContext());
 
   //Collect the contents of unnamed namespaces
@@ -1687,8 +1807,34 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
     
   decls = Cpp::hideOverloadedDeclarations(decls, typeIsConst);
 
-  foreach( const DeclarationDepthPair& decl, decls )
-    items << CompletionTreeItemPointer( new NormalDeclarationCompletionItem(DeclarationPointer(decl.first), KDevelop::CodeCompletionContext::Ptr(this), decl.second ) );
+  QList<IndexedType> matchTypes;
+  if (m_parentContext)
+    matchTypes = parentContext()->matchTypes();
+  QList<CompletionTreeItemPointer> lookaheadMatches;
+  foreach( const DeclarationDepthPair& decl, decls ) {
+    NormalDeclarationCompletionItem* item = new NormalDeclarationCompletionItem(DeclarationPointer(decl.first), KDevelop::CodeCompletionContext::Ptr(this), decl.second );
+
+    if( m_onlyShow == ShowIntegralConstants && !isIntegralConstant(decl.first, false) )
+      item->m_fixedMatchQuality = 0;
+
+    items << CompletionTreeItemPointer(item);
+
+    if (!matchTypes.size())
+      continue;
+
+    QList<DeclAccessPair> lookaheadDecls = getLookaheadMatches(decl.first, matchTypes);
+    foreach(const DeclAccessPair &lookaheadDecl, lookaheadDecls)
+    {
+      NormalDeclarationCompletionItem* lookaheadItem =
+          new NormalDeclarationCompletionItem(DeclarationPointer(lookaheadDecl.first), KDevelop::CodeCompletionContext::Ptr(this));
+      lookaheadItem->prefixText = decl.first->identifier().toString() + (lookaheadDecl.second ? "->" : ".");
+      //Perhaps it'd be nice to have these stand out more without polluting the "Best Matches"
+      //NormalDeclarationCompletionItem should be refactored in order to make subclassing simpler
+      lookaheadItem->m_fixedMatchQuality = 0;
+      lookaheadMatches << CompletionTreeItemPointer(lookaheadItem);
+    }
+  }
+  eventuallyAddGroup(i18n("Lookahead Matches"), 100, lookaheadMatches);
 
   ///Eventually show additional specificly known items for the matched argument-type, like for example enumerators for enum types
   CodeCompletionContext* parent = parentContext();
@@ -1704,7 +1850,7 @@ QList< CompletionTreeItemPointer > CodeCompletionContext::standardAccessCompleti
         }
       }
     }
-    else if (parent->accessType() == BinaryOpFunctionCallAccess)
+    else if (parent->accessType() == BinaryOpFunctionCallAccess || parent->accessType() == CaseAccess)
     {
       items += specialItemsForArgumentType(parent->m_expressionResult.type.abstractType());
     }
@@ -1735,7 +1881,7 @@ void CodeCompletionContext::addImplementationHelpers()
 {
   QList<CompletionTreeItemPointer> helpers = getImplementationHelpers();
   if(!helpers.isEmpty()) {
-    eventuallyAddGroup(i18n("Implement Function"), 0, helpers);
+    eventuallyAddGroup(i18nc("@action", "Implement Function"), 0, helpers);
   }
 }
 
@@ -1788,6 +1934,11 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
     if(!m_valid)
       return items;
 
+    // Call parent context before adding our items because if parent is CaseAccess, this call
+    // will make it compute its expression type (which we need in standardAccessCompletionItems())
+    if(shouldAddParentItems(fullCompletion))
+      items = parentContext()->completionItems( shouldAbort, fullCompletion );
+
     switch(m_accessType) {
       case MemberAccess:
       case ArrowMemberAccess:
@@ -1821,7 +1972,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
         if(parentContext() && parentContext()->m_knownArgumentExpressions.size() != 2)
           break;
       default:
-        if(depth() == 0 && (m_onlyShow == ShowAll || m_onlyShow == ShowTypes))
+        if(depth() == 0 && (m_onlyShow == ShowAll || m_onlyShow == ShowTypes || m_onlyShow == ShowIntegralConstants))
         {
           items += standardAccessCompletionItems();
           LOCKDUCHAIN; if (!m_duContext) return items;
@@ -1833,9 +1984,6 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
         break;
     }
 
-    if(shouldAddParentItems(fullCompletion))
-      items = parentContext()->completionItems( shouldAbort, fullCompletion ) + items;
-
     if(depth() == 0)
     {
       LOCKDUCHAIN; if (!m_duContext) return items;
@@ -1909,39 +2057,20 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::getImplementationHelpers
   return ret;
 }
 
-QualifiedIdentifier CodeCompletionContext::requiredPrefix(Declaration* decl) const {
-  QualifiedIdentifier worstCase = decl->context()->scopeIdentifier(true);
-  if(!m_duContext)
-    return worstCase;
-  QualifiedIdentifier currentPrefix;
-
-  while(1) {
-    QList<Declaration*> found = m_duContext->findDeclarations( currentPrefix + decl->identifier() );
-    if(found.contains(decl))
-      return currentPrefix;
-
-    if(currentPrefix.count() >= worstCase.count()) {
-      return worstCase;
-    }else {
-      currentPrefix.push(worstCase.at(currentPrefix.count()));
-    }
-  }
-}
-
 QList< KSharedPtr< KDevelop::CompletionTreeItem > > CodeCompletionContext::specialItemsForArgumentType(TypePtr< KDevelop::AbstractType > type) {
   QList< KSharedPtr< KDevelop::CompletionTreeItem > > items;
   if(EnumerationType::Ptr enumeration = TypeUtils::realType(type, m_duContext->topContext()).cast<EnumerationType>()) {
     Declaration* enumDecl = enumeration->declaration(m_duContext->topContext());
     if(enumDecl && enumDecl->internalContext()) {
 
-      QualifiedIdentifier prefix = requiredPrefix(enumDecl);
+      // This is necessary to avoid duplication of locally visible enumerators, which also get added by standardAccessCompletionItems().
+      if( visibleFromWithin( enumDecl, m_duContext.data() ) )
+        return items;
 
       DUContext* enumInternal = enumDecl->internalContext();
       foreach(Declaration* enumerator, enumInternal->localDeclarations()) {
-        QualifiedIdentifier id = prefix + enumerator->identifier();
         items << CompletionTreeItemPointer(new NormalDeclarationCompletionItem( DeclarationPointer(enumerator), KDevelop::CodeCompletionContext::Ptr(this), 0 ));
-        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->alternativeText = id.toString();
-        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->useAlternativeText = true;
+        static_cast<NormalDeclarationCompletionItem*>(items.back().data())->prependScopePrefix = true;
       }
     }
   }
@@ -1957,6 +2086,52 @@ bool CodeCompletionContext::visibleFromWithin(KDevelop::Declaration* decl, DUCon
   return visibleFromWithin(decl, currentContext->parentContext());
 }
 
+bool CodeCompletionContext::isIntegralConstant(Declaration* decl, bool acceptHelperItems) {
+
+    // Usability issue: if we're matching for integral constants,
+    // types and functions are also allowed (see filterDeclaration()),
+    // but shall not pollute "best matches" as one rarely would need them.
+    // So introduce "acceptHelperItems" to distinguish between filtering items and demoting them to zero match quality.
+    // (see standardAccessCompletionItems())
+
+    switch (decl->kind()) {
+      case Declaration::Namespace:
+      case Declaration::NamespaceAlias:
+      case Declaration::Type:
+        // Type-names and namespace-names in general may be used for completing constants either as:
+        // "IntegralType(42)"
+        // "EnumName::someEnumerator" (that's valid C++11)
+        // "ClassName::someStaticConstantField"
+        // "NamespaceName::see_everything_above"
+        return acceptHelperItems;
+
+      case Declaration::Instance: {
+        FunctionType::Ptr funType;
+        IntegralType::Ptr integralType;
+
+        // If a declaration is a known integer compile-time constant, it's valid.
+        // NOTE: are there any chances of missing a compile-time constant due to our parser's shortcomings?
+        if (ConstantIntegralType::Ptr constantIntegralType = decl->type<ConstantIntegralType>())
+          integralType = constantIntegralType.cast<IntegralType>();
+
+        // If a declaration is a constexpr function returning integral type, it's valid.
+        // TODO: change this when constexpr becomes parsed:
+        // - add check for constexpr-ness after (funType = ...)
+        // - remove (acceptHelperItems) since that will be a true match - eligible for "best matches"
+        else if (acceptHelperItems && (funType = decl->type<FunctionType>()))
+          integralType = funType->returnType().cast<IntegralType>();
+
+        // Finally, check if retrieved type is integer.
+        return (integralType && TypeUtils::isIntegerType(integralType));
+      }
+
+      case Declaration::Alias:
+      case Declaration::Import:
+      default:
+        return false;
+    }
+}
+
 /**
  * see @p type as function type and try to get it's return type as IntegralType data type.
  */
@@ -1977,6 +2152,10 @@ bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* dec
   if(!decl)
     return true;
 
+  if (decl->isExplicitlyDeleted()) {
+    return false;
+  }
+
   if(dynamic_cast<TemplateParameterDeclaration*>(decl) && !visibleFromWithin(decl, m_duContext.data()))
     return false;
   
@@ -2007,6 +2186,12 @@ bool  CodeCompletionContext::filterDeclaration(Declaration* decl, DUContext* dec
     }
   }
 
+  if (m_accessType == NamespaceAccess)
+    return decl->kind() == Declaration::Namespace || decl->kind() == Declaration::NamespaceAlias;
+
+  if(m_onlyShow == ShowIntegralConstants && !isIntegralConstant(decl, true))
+    return false;
+
   if(m_onlyShow == ShowTypes && decl->kind() != Declaration::Type && decl->kind() != Declaration::Namespace
      && decl->kind() != Declaration::NamespaceAlias )
     return false;
