@@ -21,15 +21,12 @@
 #include "contextbuilder.h"
 
 
-#include <ktexteditor/smartrange.h>
-#include <ktexteditor/smartinterface.h>
 #include <ktexteditor/document.h>
 
 #include <language/duchain/duchain.h>
 #include <language/duchain/topducontext.h>
 #include <language/duchain/declaration.h>
 #include <language/duchain/use.h>
-#include <language/duchain/smartconverter.h>
 #include <util/pushvalue.h>
 
 #include "parsesession.h"
@@ -57,7 +54,7 @@ bool containsContext( const QList<LineContextPair>& lineContexts, TopDUContext*
 
 bool importsContext( const QList<LineContextPair>& lineContexts, TopDUContext* context ) {
   foreach( const LineContextPair& ctx, lineContexts )
-    if( ctx.context && ctx.context->imports(context, KDevelop::SimpleCursor()) )
+    if( ctx.context && ctx.context->imports(context, KDevelop::CursorInRevision()) )
       return true;
   return false;
 }
@@ -96,46 +93,19 @@ void getFirstLast(AST** first, AST** last, const ListNode<_Tp> *nodes)
   while (it != end);
 }
 
-
-ContextBuilder::ContextBuilder ()
-  : m_nameCompiler(0)
-  , m_inFunctionDefinition(false)
-  , m_templateDeclarationDepth(0)
-  , m_typeSpecifierWithoutInitDeclarators((uint)-1)
-  , m_onlyComputeVisible(false)
-  , m_onlyComputeSimplified(false)
-  , m_computeEmpty(false)
-  , m_currentInitializer(0)
-  , m_mapAst(false)
-{
-}
-
 ContextBuilder::ContextBuilder (ParseSession* session)
-  : m_nameCompiler(0)
-  , m_inFunctionDefinition(false)
+:   m_inFunctionDefinition(false)
+  , m_editor(session)
+  , m_nameCompiler(session)
   , m_templateDeclarationDepth(0)
   , m_typeSpecifierWithoutInitDeclarators((uint)-1)
   , m_onlyComputeVisible(false)
   , m_onlyComputeSimplified(false)
   , m_computeEmpty(false)
   , m_currentInitializer(0)
+  , m_currentCondition(0)
   , m_mapAst(false)
 {
-  setEditor(new CppEditorIntegrator(session), true);
-}
-
-ContextBuilder::ContextBuilder (CppEditorIntegrator* editor)
-  : m_nameCompiler(0)
-  , m_inFunctionDefinition(false)
-  , m_templateDeclarationDepth(0)
-  , m_typeSpecifierWithoutInitDeclarators((uint)-1)
-  , m_onlyComputeVisible(false)
-  , m_onlyComputeSimplified(false)
-  , m_computeEmpty(false)
-  , m_currentInitializer(0)
-  , m_mapAst(false)
-{
-  setEditor(editor, false);
 }
 
 void ContextBuilder::setOnlyComputeVisible(bool onlyVisible) {
@@ -158,7 +128,7 @@ void ContextBuilder::createUserProblem(AST* node, QString text) {
     KDevelop::ProblemPointer problem(new KDevelop::Problem);
     problem->setDescription(text);
     problem->setSource(KDevelop::ProblemData::DUChainBuilder);
-    problem->setFinalLocation(DocumentRange(HashedString(currentContext()->url().str()), editor()->findRange(node).textRange()));
+    problem->setFinalLocation(DocumentRange(IndexedString(currentContext()->url().str()), editor()->findRange(node).castToSimpleRange()));
     currentContext()->topContext()->addProblem(problem);
 }
 
@@ -188,13 +158,6 @@ void ContextBuilder::addBaseType( KDevelop::BaseClassInstance base, BaseSpecifie
   }
 }
 
-void ContextBuilder::setEditor(CppEditorIntegrator* editor, bool ownsEditorIntegrator)
-{
-  ContextBuilderBase::setEditor(editor, ownsEditorIntegrator);
-  m_nameCompiler = new NameCompiler(editor->parseSession());
-}
-
-
 void addImportedParentContextSafely(DUContext* context, DUContext* import) {
   if(import->imports(context)) {
     kDebug() << "prevented endless recursive import";
@@ -234,22 +197,21 @@ DUContext* ContextBuilder::contextFromNode( AST* node )
   return node->ducontext;
 }
 
-KTextEditor::Range ContextBuilder::editorFindRange( AST* fromRange, AST* toRange )
+RangeInRevision ContextBuilder::editorFindRange( AST* fromRange, AST* toRange )
 {
-  return editor()->findRange(fromRange, toRange).textRange();
+  return editor()->findRange(fromRange, toRange);
 }
 
-KTextEditor::Range ContextBuilder::editorFindRangeForContext( AST* fromRange, AST* toRange )
+RangeInRevision ContextBuilder::editorFindRangeForContext( AST* fromRange, AST* toRange )
 {
-  return editor()->findRangeForContext(fromRange->start_token, toRange->end_token).textRange();
+  return editor()->findRangeForContext(fromRange->start_token, toRange->end_token);
 }
 
 ContextBuilder::~ContextBuilder ()
 {
-  delete m_nameCompiler;
 }
 
-QPair<DUContext*, QualifiedIdentifier> ContextBuilder::findPrefixContext(const QualifiedIdentifier& id, KDevelop::SimpleCursor pos) {
+QPair<DUContext*, QualifiedIdentifier> ContextBuilder::findPrefixContext(const QualifiedIdentifier& id, KDevelop::CursorInRevision pos) {
   if(id.count() < 2)
     return qMakePair((DUContext*)0, QualifiedIdentifier());
   
@@ -270,7 +232,6 @@ QPair<DUContext*, QualifiedIdentifier> ContextBuilder::findPrefixContext(const Q
       if(classContext && classContext->type() == DUContext::Class) {
         import = classContext;
         //Change the prefix-id so it respects namespace-imports
-        
         prefixId = classContext->scopeIdentifier(true);
         if(prefixId.count() >= currentScopeId.count() && prefixId.left(currentScopeId.count()) == currentScopeId)
           prefixId = prefixId.mid(currentScopeId.count());
@@ -283,7 +244,7 @@ QPair<DUContext*, QualifiedIdentifier> ContextBuilder::findPrefixContext(const Q
   return qMakePair(import, prefixId);
 }
 
-void ContextBuilder::openPrefixContext(AST* ast, const QualifiedIdentifier& id, const SimpleCursor& pos) {
+void ContextBuilder::openPrefixContext(AST* ast, const QualifiedIdentifier& id, const CursorInRevision& pos) {
   if(id.count() < 2)
     return;
 
@@ -339,9 +300,6 @@ KDevelop::TopDUContext* ContextBuilder::buildProxyContextFromContent(Cpp::Enviro
   Cpp::EnvironmentFile* filePtr = const_cast<Cpp::EnvironmentFile*>(file.data() );
   
   filePtr->setIsProxyContext(true);
-  
-  //Never give smart-ranges to proxy-contexts
-  editor()->setCurrentUrl(file->url(), false);
 
   TopDUContext* topLevelContext = 0;
   {
@@ -360,7 +318,7 @@ KDevelop::TopDUContext* ContextBuilder::buildProxyContextFromContent(Cpp::Enviro
     } else {
       kDebug(9007) << "ContextBuilder::buildProxyContextFromContent: compiling";
 
-      topLevelContext = new CppDUContext<TopDUContext>(editor()->currentUrl(), SimpleRange(), filePtr);
+      topLevelContext = new CppDUContext<TopDUContext>(file->url(), RangeInRevision(), filePtr);
       topLevelContext->setType(DUContext::Global);
 
       Q_ASSERT(dynamic_cast<CppDUContext<TopDUContext>* >(topLevelContext));
@@ -371,15 +329,12 @@ KDevelop::TopDUContext* ContextBuilder::buildProxyContextFromContent(Cpp::Enviro
       topLevelContext->updateImportsCache(); //Mark that we will use a cached import-structure
     }
 
-    if(content) {
-      cppContext->clearImportedParentContexts();
-      cppContext->addImportedParentContext(content.data());
-      cppContext->updateImportsCache(); //Mark that we will use a cached import-structure
-    } else {
-      ///This happens if a content-context is deleted from the du-chain during the time that the du-chain is not locked by this thread
-      kDebug(9007) << "ContextBuilder::buildProxyContextFromContent: Content-context lost for " << file->url().str();
-      Q_ASSERT(0);
-    }
+    Q_ASSERT(content);
+    
+    cppContext->clearImportedParentContexts();
+    cppContext->addImportedParentContext(content.data());
+    cppContext->updateImportsCache(); //Mark that we will use a cached import-structure
+
     Q_ASSERT(topLevelContext->importedParentContexts().count());
   }
 
@@ -398,46 +353,27 @@ ReferencedTopDUContext ContextBuilder::buildContexts(Cpp::EnvironmentFilePointer
       updateContext->parsingEnvironmentFile()->setIsProxyContext(false);
     }
   }
-
-  if(editor()->currentUrl() != file->url())
-    editor()->setCurrentUrl(file->url(), true);
-
+  
   ReferencedTopDUContext topLevelContext;
   {
     DUChainWriteLocker lock(DUChain::lock());
     topLevelContext = updateContext;
 
-    if( topLevelContext && topLevelContext->smartRange() && !(topLevelContext->parsingEnvironmentFile() && topLevelContext->parsingEnvironmentFile()->isProxyContext()))
-      if (topLevelContext->smartRange()->parentRange()) { //Top-range must have no parent, else something is wrong with the structure
-        kWarning() << *topLevelContext->smartRange() << "erroneously has a parent range" << *topLevelContext->smartRange()->parentRange();
-        Q_ASSERT(false);
-      }
-
+    RangeInRevision topRange = RangeInRevision(CursorInRevision(0,0), CursorInRevision(INT_MAX, INT_MAX));
+    
     if (topLevelContext) {
       kDebug(9007) << "ContextBuilder::buildContexts: recompiling";
       setRecompiling(true);
-
-      if (compilingContexts()) {
-        // To here...
-        LockedSmartInterface iface = editor()->smart();
-        if (iface && topLevelContext->range().textRange() != iface.currentDocument()->documentRange()) {
-          topLevelContext->setRange(SimpleRange(iface.currentDocument()->documentRange()));
-          //This happens the whole file is deleted, and then a space inserted.
-          kDebug(9007) << "WARNING: Top-level context has wrong size: " << topLevelContext->range().textRange() << " should be: " << iface.currentDocument()->documentRange();
-        }
-      }
-
       DUChain::self()->updateContextEnvironment( topLevelContext, const_cast<Cpp::EnvironmentFile*>(file.data() ) );
+      topLevelContext->setRange(topRange);
     } else {
       kDebug(9007) << "ContextBuilder::buildContexts: compiling";
       setRecompiling(false);
 
       Q_ASSERT(compilingContexts());
 
-      LockedSmartInterface iface = editor()->smart();
-      topLevelContext = new CppDUContext<TopDUContext>(editor()->currentUrl(), iface.currentDocument() ? SimpleRange(iface.currentDocument()->documentRange()) : SimpleRange(SimpleCursor(0,0), SimpleCursor(INT_MAX, INT_MAX)), const_cast<Cpp::EnvironmentFile*>(file.data()));
+      topLevelContext = new CppDUContext<TopDUContext>(file->url(), topRange, const_cast<Cpp::EnvironmentFile*>(file.data()));
 
-      topLevelContext->setSmartRange(editor()->topRange(iface, CppEditorIntegrator::DefinitionUseChain), DocumentRangeObject::Own);
       topLevelContext->setType(DUContext::Global);
       topLevelContext->setFlags((TopDUContext::Flags)(TopDUContext::UpdatingContext | topLevelContext->flags()));
       DUChain::self()->addDocumentChain(topLevelContext);
@@ -454,13 +390,13 @@ ReferencedTopDUContext ContextBuilder::buildContexts(Cpp::EnvironmentFilePointer
             topLevelContext->removeImportedParentContext(parent.context(0));
       }
 
-      QList< QPair<TopDUContext*, SimpleCursor> > realIncluded;
-      QList< QPair<TopDUContext*, SimpleCursor> > realTemporaryIncluded;
+      QList< QPair<TopDUContext*, CursorInRevision> > realIncluded;
+      QList< QPair<TopDUContext*, CursorInRevision> > realTemporaryIncluded;
       foreach (const LineContextPair &included, *includes)
         if(!included.temporary)
-          realIncluded << qMakePair(included.context.data(), SimpleCursor(included.sourceLine, 0));
+          realIncluded << qMakePair(included.context.data(), CursorInRevision(included.sourceLine, 0));
         else
-          realTemporaryIncluded << qMakePair(included.context.data(), SimpleCursor(included.sourceLine, 0));
+          realTemporaryIncluded << qMakePair(included.context.data(), CursorInRevision(included.sourceLine, 0));
 
       topLevelContext->addImportedParentContexts(realIncluded);
       topLevelContext->addImportedParentContexts(realTemporaryIncluded, true);
@@ -472,7 +408,7 @@ ReferencedTopDUContext ContextBuilder::buildContexts(Cpp::EnvironmentFilePointer
   {
     DUChainReadLocker lock(DUChain::lock());
     //If we're debugging the current file, dump its preprocessed contents and the AST
-    ifDebugFile( HashedString(file->identity().url().str()), { kDebug() << stringFromContents(editor()->parseSession()->contentsVector()); Cpp::DumpChain dump; dump.dump(node, editor()->parseSession()); } );
+    ifDebugFile( IndexedString(file->identity().url().str()), { kDebug() << stringFromContents(editor()->parseSession()->contentsVector()); Cpp::DumpChain dump; dump.dump(node, editor()->parseSession()); } );
   }
 
   if(m_computeEmpty)
@@ -481,28 +417,20 @@ ReferencedTopDUContext ContextBuilder::buildContexts(Cpp::EnvironmentFilePointer
     DUChainWriteLocker lock(DUChain::lock());
     topLevelContext->cleanIfNotEncountered(QSet<DUChainBase*>());
   }else{
+    Q_ASSERT(node);
     node->ducontext = topLevelContext;
     supportBuild(node);
   }
 
-  {
-    LockedSmartInterface iface = editor()->smart();
-    if (iface && topLevelContext->range().textRange() != iface.currentDocument()->documentRange()) {
-      kDebug(9007) << "WARNING: Top-level context has wrong size: " << topLevelContext->range().textRange() << " should be: " << iface.currentDocument()->documentRange();
-      topLevelContext->setRange( SimpleRange(iface.currentDocument()->documentRange()) );
-    }
-  }
-
   {
     DUChainReadLocker lock(DUChain::lock());
 
     kDebug(9007) << "built top-level context with" << topLevelContext->localDeclarations().size() << "declarations and" << topLevelContext->importedParentContexts().size() << "included files";
     //If we're debugging the current file, dump the du-chain and the smart ranges
-    ifDebugFile( HashedString(file->identity().url().str()), { KDevelop::DumpChain dump; dump.dump(topLevelContext); if(topLevelContext->smartRange()) dump.dumpRanges(topLevelContext->smartRange()); } );
+    ifDebugFile( IndexedString(file->identity().url().str()), { KDevelop::dumpDUContext(topLevelContext); } );
 
 /*     if( m_recompiling ) {
-      DumpChain dump;
-      dump.dump(topLevelContext);
+      dumpDUContext(topLevelContext);
       kDebug(9007) << dump.dotGraph(topLevelContext);
      }*/
   }
@@ -523,41 +451,6 @@ ReferencedTopDUContext ContextBuilder::buildContexts(Cpp::EnvironmentFilePointer
   return topLevelContext;
 }
 
-// KDevelop::DUContext* ContextBuilder::buildSubContexts(const HashedString& url, AST *node, KDevelop::DUContext* parent) {
-//   setCompilingContexts(true);
-//   setRecompiling(false);
-//
-//   editor()->setCurrentUrl(url);
-//
-//   node->ducontext = parent;
-//
-//   {
-//     //copied out of supportBuild
-//
-//     openContext(node->ducontext);
-//
-//     editor()->setCurrentRange(editor()->topRange(EditorIntegrator::DefinitionUseChain));
-//
-//     visit (node);
-//
-//     closeContext();
-//   }
-//
-//   setCompilingContexts(false);
-//
-//   if( node->ducontext == parent ) {
-//     //The node's du-context should have been replaced!
-//     //Maybe dump the node
-//     kDebug(9007) << "Error in ContextBuilder::buildSubContexts(...): du-context was not replaced with new one";
-//     DUChainWriteLocker lock(DUChain::lock());
-//     delete node->ducontext;
-//
-//     node->ducontext = 0;
-//   }
-//
-//   return node->ducontext;
-// }
-
 void ContextBuilder::visitNamespace (NamespaceAST *node)
 {
   QualifiedIdentifier identifier;
@@ -587,8 +480,9 @@ void ContextBuilder::visitEnumSpecifier(EnumSpecifierAST* node)
   if(m_onlyComputeSimplified)
     return;
   
-  //The created context must be unnamed, so the enumerators can be found globally with the correct scope
-  openContext(node, DUContext::Enum, 0 );
+  //The created context must be unnamed if not an "enum class", so the enumerators can be found globally with the correct scope.
+  //In case of an "enum class" this enum creates its own context.
+  openContext(node, DUContext::Enum, node->isClass ? node->name : 0 );
 
   {
     DUChainWriteLocker lock(DUChain::lock());
@@ -653,11 +547,11 @@ void ContextBuilder::visitTypedef (TypedefAST *node)
 
 void ContextBuilder::visitFunctionDefinition (FunctionDefinitionAST *node)
 {
-  PushValue<bool> push(m_inFunctionDefinition, (bool)node->function_body);
+  PushValue<bool> push(m_inFunctionDefinition, (bool)node->function_body || node->defaultDeleted != FunctionDefinitionAST::NotDefaultOrDeleted);
 
   QualifiedIdentifier functionName;
-  if (compilingContexts() && node->init_declarator && node->init_declarator->declarator && node->init_declarator->declarator->id) {
-    identifierForNode(node->init_declarator->declarator->id, functionName);
+  if (compilingContexts() && node->declarator && node->declarator->id) {
+    identifierForNode(node->declarator->id, functionName);
 
     if (functionName.count() >= 2) {
       
@@ -711,7 +605,7 @@ void ContextBuilder::visitFunctionDefinition (FunctionDefinitionAST *node)
 void ContextBuilder::visitFunctionDeclaration (FunctionDefinitionAST* node)
 {
   visit(node->type_specifier);
-  visit(node->init_declarator);
+  visit(node->declarator);
 }
 
 DUContext* ContextBuilder::openContextEmpty(AST* rangeNode, DUContext::ContextType type)
@@ -720,7 +614,7 @@ DUContext* ContextBuilder::openContextEmpty(AST* rangeNode, DUContext::ContextTy
 #ifdef DEBUG_UPDATE_MATCHING
     kDebug() << "opening context with text" << editor()->tokensToStrings( rangeNode->start_token, rangeNode->end_token );
 #endif
-    KDevelop::SimpleRange range = editor()->findRangeForContext(rangeNode->start_token, rangeNode->end_token);
+    KDevelop::RangeInRevision range = editor()->findRangeForContext(rangeNode->start_token, rangeNode->end_token);
     range.end = range.start;
     DUContext* ret = openContextInternal(range, type, QualifiedIdentifier());
     rangeNode->ducontext = ret;
@@ -728,15 +622,11 @@ DUContext* ContextBuilder::openContextEmpty(AST* rangeNode, DUContext::ContextTy
 
   } else {
     openContext(rangeNode->ducontext);
-    {
-      LockedSmartInterface iface = editor()->smart();
-      editor()->setCurrentRange(iface, currentContext()->smartRange());
-    }
     return currentContext();
   }
 }
 
-DUContext* ContextBuilder::openContextInternal(const KDevelop::SimpleRange& range, DUContext::ContextType type, const QualifiedIdentifier& identifier)
+DUContext* ContextBuilder::openContextInternal(const KDevelop::RangeInRevision& range, DUContext::ContextType type, const QualifiedIdentifier& identifier)
 {
   DUContext* ret = ContextBuilderBase::openContextInternal(range, type, identifier);
 
@@ -755,7 +645,7 @@ DUContext* ContextBuilder::openContextInternal(const KDevelop::SimpleRange& rang
   return ret;
 }
 
-DUContext* ContextBuilder::newContext(const SimpleRange& range)
+DUContext* ContextBuilder::newContext(const RangeInRevision& range)
 {
   return new CppDUContext<DUContext>(range, currentContext());
 }
@@ -763,7 +653,7 @@ DUContext* ContextBuilder::newContext(const SimpleRange& range)
 #ifdef DEBUG_CONTEXT_RANGES
 void ContextBuilder::checkRanges()
 {
-  for(QHash<KDevelop::DUContext*, KDevelop::SimpleRange>::iterator it = m_contextRanges.begin(); it != m_contextRanges.end(); ) {
+  for(QHash<KDevelop::DUContext*, KDevelop::RangeInRevision>::iterator it = m_contextRanges.begin(); it != m_contextRanges.end(); ) {
     if(it.key()->range() != *it) {
       kDebug(9007) << "Range of" << it.key()->scopeIdentifier(true).toString() << "changed from" << (*it).textRange() << "to" << it.key()->range().textRange() << "at\n" << kBacktrace();
       it = m_contextRanges.erase(it); //Remove it so we see each change only once
@@ -837,7 +727,7 @@ class VerifyExpressionVisitor : public Cpp::ExpressionVisitor {
 class IdentifierVerifier : public DefaultVisitor
 {
 public:
-  IdentifierVerifier(ContextBuilder* _builder, const SimpleCursor& _cursor)
+  IdentifierVerifier(ContextBuilder* _builder, const CursorInRevision& _cursor)
     : builder(_builder)
     , result(true)
     , cursor(_cursor)
@@ -846,7 +736,7 @@ public:
 
   ContextBuilder* builder;
   bool result; //Will be true when this should be an expression, else false.
-  SimpleCursor cursor;
+  CursorInRevision cursor;
 
   void visitPostfixExpression(PostfixExpressionAST* node)
   {
@@ -910,7 +800,7 @@ void ContextBuilder::visitExpressionOrDeclarationStatement(ExpressionOrDeclarati
 
         node->expression->ducontext = currentContext();
         iv.parse(node->expression);*/
-        IdentifierVerifier iv(this, SimpleCursor(editor()->findPosition(node->start_token)));
+        IdentifierVerifier iv(this, CursorInRevision(editor()->findPosition(node->start_token)));
         iv.visit(node->expression);
         //kDebug(9007) << editor()->findPosition(node->start_token) << "IdentifierVerifier returned" << iv.result;
         node->expressionChosen = iv.result;
@@ -930,6 +820,8 @@ void ContextBuilder::visitForStatement(ForStatementAST *node)
 {
   // Not setting the member var because it gets nuked in visitSimpleDeclaration
   AST* first = node->init_statement;
+  if (!first)
+    first = node->range_declaration;
   if (!first)
     first = node->condition;
   if (!first)
@@ -940,14 +832,25 @@ void ContextBuilder::visitForStatement(ForStatementAST *node)
   AST* second = node->expression;
   if (!second)
     second = node->condition;
+  if (!second)
+    second = node->range_declaration;
   if (!second)
     second = node->init_statement;
 
   DUContext* secondParentContext = openContext(first, second, DUContext::Other);
 
-  visit(node->init_statement);
-  visit(node->condition);
-  visit(node->expression);
+  if (node->range_declaration) {
+    // in range-based for we first visit the expression
+    // since it might define the type for cases like
+    // for(auto i : foo)
+    handleRangeBasedFor(node->expression, node->range_declaration);
+    Q_ASSERT(!node->init_statement);
+    Q_ASSERT(!node->condition);
+  } else {
+    visit(node->init_statement);
+    visit(node->condition);
+    visit(node->expression);
+  }
 
   closeContext();
 
@@ -964,17 +867,23 @@ void ContextBuilder::visitForStatement(ForStatementAST *node)
   m_importedParentContexts.clear();
 }
 
-void ContextBuilder::createTypeForInitializer(InitializerAST* /*node*/) {
+void ContextBuilder::handleRangeBasedFor(ExpressionAST* container, ForRangeDeclarationAst* iterator)
+{
+  visit(container);
+  visit(iterator);
 }
 
-void ContextBuilder::closeTypeForInitializer(InitializerAST* /*node*/) {
-}
+void ContextBuilder::createTypeForDeclarator(DeclaratorAST* /*node*/)
+{ }
 
-void ContextBuilder::createTypeForDeclarator(DeclaratorAST* /*node*/) {
-}
+void ContextBuilder::closeTypeForDeclarator(DeclaratorAST* /*node*/)
+{ }
 
-void ContextBuilder::closeTypeForDeclarator(DeclaratorAST* /*node*/) {
-}
+void ContextBuilder::createTypeForInitializer(InitializerAST* /*node*/)
+{ }
+
+void ContextBuilder::createTypeForCondition(ConditionAST* /*node*/)
+{ }
 
 void ContextBuilder::visitParameterDeclarationClause(ParameterDeclarationClauseAST* node)
 {
@@ -990,9 +899,9 @@ void ContextBuilder::visitParameterDeclarationClause(ParameterDeclarationClauseA
 void ContextBuilder::visitInitDeclarator(InitDeclaratorAST *node)
 {
   QualifiedIdentifier id;
-  if(node->declarator && node->declarator->id && node->declarator->id->qualified_names && (!node->declarator->parameter_declaration_clause || node->declarator->parameter_is_initializer)) {
+  if(node->declarator && node->declarator->id && node->declarator->id->qualified_names && !node->declarator->parameter_declaration_clause) {
     //Build a prefix-context for external variable-definitions
-    SimpleCursor pos = editor()->findPosition(node->start_token, KDevelop::EditorIntegrator::FrontEdge);
+    CursorInRevision pos = editor()->findPosition(node->start_token, CppEditorIntegrator::FrontEdge);
     identifierForNode(node->declarator->id, id);
     
     openPrefixContext(node, id, pos);
@@ -1021,9 +930,13 @@ void ContextBuilder::visitDeclarator(DeclaratorAST *node) {
     return;
 
   createTypeForDeclarator(node);
-  
-  if(m_currentInitializer) //Needs to be visited now, so the type-builder can use the initializer to build a constant integral tyoe
-    createTypeForInitializer(m_currentInitializer); 
+
+  // These need to be visited now, so the type-builder can use them
+  // to build a constant integral types
+  if(m_currentInitializer)
+    createTypeForInitializer(m_currentInitializer);
+  else if(m_currentCondition)
+    createTypeForCondition(m_currentCondition);
 
   if (node->parameter_declaration_clause && (compilingContexts() || node->parameter_declaration_clause->ducontext)) {
     DUContext* ctx = openContext(node->parameter_declaration_clause, DUContext::Function, node->id);
@@ -1036,11 +949,9 @@ void ContextBuilder::visitDeclarator(DeclaratorAST *node) {
   visitNodes(this, node->array_dimensions);
   visit(node->parameter_declaration_clause);
   visit(node->exception_spec);
+  visit(node->trailing_return_type);
   //END Finished with default visitor
 
-  if(m_currentInitializer)
-    closeTypeForInitializer(m_currentInitializer);
-  
   closeTypeForDeclarator(node);
 
   if (node->parameter_declaration_clause && (compilingContexts() || node->parameter_declaration_clause->ducontext))
@@ -1109,6 +1020,24 @@ void ContextBuilder::visitIfStatement(IfStatementAST* node)
   }
 }
 
+void ContextBuilder::visitSwitchStatement(SwitchStatementAST* node)
+{
+  DUContext* secondParentContext = openContext(node->condition, DUContext::Other);
+
+  visit(node->condition);
+
+  closeContext();
+
+  if (node->statement) {
+    const bool contextNeeded = createContextIfNeeded(node->statement, secondParentContext);
+
+    visit(node->statement);
+
+    if (contextNeeded)
+      closeContext();
+  }
+}
+
 void ContextBuilder::visitDoStatement(DoStatementAST *node)
 {
   if(!node->statement) {
@@ -1190,6 +1119,29 @@ void ContextBuilder::visitCatchStatement(CatchStatementAST *node)
   }
 }
 
+void ContextBuilder::visitLambdaDeclarator(LambdaDeclaratorAST* node)
+{
+  if (node->parameter_declaration_clause) {
+    DUContext* ctx = openContext(node->parameter_declaration_clause, DUContext::Function);
+    addImportedContexts();
+    if(compilingContexts())
+      queueImportedContext(ctx);
+  }
+
+  DefaultVisitor::visitLambdaDeclarator(node);
+
+  if (node->parameter_declaration_clause) {
+    closeContext();
+  }
+}
+
+void ContextBuilder::visitCondition(ConditionAST* node)
+{
+  m_currentCondition = node;
+  DefaultVisitor::visitCondition(node);
+  m_currentCondition = 0;
+}
+
 bool ContextBuilder::createContextIfNeeded(AST* node, DUContext* importedParentContext)
 {
   QVector<DUContext::Import> imports;
@@ -1219,11 +1171,7 @@ void ContextBuilder::identifierForNode(NameAST* id, TypeSpecifierAST** typeSpeci
     target = QualifiedIdentifier();
   }
 
-  m_nameCompiler->run(id, &target);
+  m_nameCompiler.run(id, &target);
   if( typeSpecifier )
-    *typeSpecifier = m_nameCompiler->lastTypeSpecifier();
-}
-
-bool ContextBuilder::smart() const {
-  return editor()->smart();
+    *typeSpecifier = m_nameCompiler.lastTypeSpecifier();
 }
