@@ -39,7 +39,12 @@ const bool allowADL = true;
 // uncomment to get debugging info on ADL - very expensive on parsing
 // #define DEBUG_ADL
 
-OverloadResolver::OverloadResolver( DUContextPointer context, TopDUContextPointer topContext, bool forceIsInstance ) : m_context( context ), m_topContext( topContext ), m_worstConversionRank( NoMatch ), m_forceIsInstance( forceIsInstance )
+OverloadResolver::OverloadResolver( DUContextPointer context, TopDUContextPointer topContext, Constness constness, bool forceIsInstance )
+: m_context( context )
+, m_topContext( topContext )
+, m_worstConversionRank( NoMatch )
+, m_forceIsInstance( forceIsInstance )
+, m_constness(constness)
 {
 
 }
@@ -64,7 +69,7 @@ Declaration* OverloadResolver::resolveConstructor( const ParameterList& params,
 
       if ( functionDeclaration /*&& functionDeclaration->isConstructor()*/ ) //Test not needed, because name == classname
       {
-        if ( function->indexedArgumentsSize() >= params.parameters.size() )
+        if ( function->indexedArgumentsSize() >= static_cast<uint>(params.parameters.size()) )
         {
           if ( !implicit || !functionDeclaration->isExplicit() )
             goodDeclarations << *it;
@@ -113,15 +118,14 @@ void OverloadResolver::expandDeclarations( const QList<Declaration*>& declaratio
   for ( QList<Declaration*>::const_iterator it = declarations.constBegin(); it != declarations.constEnd(); ++it )
   {
     Declaration* decl = *it;
-    bool isConstant = false;
 
-    if ( CppClassType::Ptr klass = TypeUtils::realType( decl->abstractType(), m_topContext.data(), &isConstant ).cast<CppClassType>() )
+    if ( CppClassType::Ptr klass = TypeUtils::realType( decl->abstractType(), m_topContext.data() ).cast<CppClassType>() )
     {
       if ( decl->kind() == Declaration::Instance || m_forceIsInstance )
       {
         //Instances of classes should be substituted with their operator() members
         QList<Declaration*> decls;
-        TypeUtils::getMemberFunctions( klass, m_topContext.data(), decls, "operator()", isConstant );
+        TypeUtils::getMemberFunctions( klass, m_topContext.data(), decls, "operator()", klass->modifiers() & AbstractType::ConstModifier );
 
         foreach( Declaration* decl, decls )
         newDeclarations.insert( decl );
@@ -148,15 +152,14 @@ void OverloadResolver::expandDeclarations( const QList<QPair<OverloadResolver::P
   for ( QList<QPair<OverloadResolver::ParameterList, Declaration*> >::const_iterator it = declarations.constBegin(); it != declarations.constEnd(); ++it )
   {
     QPair<OverloadResolver::ParameterList, Declaration*> decl = *it;
-    bool isConstant = false;
 
-    if ( CppClassType::Ptr klass = TypeUtils::realType( decl.second->abstractType(), m_topContext.data(), &isConstant ).cast<CppClassType>() )
+    if ( CppClassType::Ptr klass = TypeUtils::realType( decl.second->abstractType(), m_topContext.data() ).cast<CppClassType>() )
     {
       if ( decl.second->kind() == Declaration::Instance || m_forceIsInstance )
       {
         //Instances of classes should be substituted with their operator() members
         QList<Declaration*> functions;
-        TypeUtils::getMemberFunctions( klass, m_topContext.data(), functions, "operator()", isConstant );
+        TypeUtils::getMemberFunctions( klass, m_topContext.data(), functions, "operator()", klass->modifiers() & AbstractType::ConstModifier );
         foreach( Declaration* f, functions )
         newDeclarations.insert( f, decl.first );
       }
@@ -198,7 +201,7 @@ Declaration* OverloadResolver::resolveList( const ParameterList& params, const Q
     if ( !decl )
       continue;
 
-    ViableFunction viable( m_topContext.data(), decl, noUserDefinedConversion );
+    ViableFunction viable( m_topContext.data(), decl, m_constness, noUserDefinedConversion );
     viable.matchParameters( params );
 
     if ( viable.isBetter( bestViableFunction ) )
@@ -238,8 +241,11 @@ QList< ViableFunction > OverloadResolver::resolveListOffsetted( const ParameterL
     Declaration* decl = applyImplicitTemplateParameters( mergedParams, it.key() );
     if ( !decl )
       continue;
-    
-    ViableFunction viable( m_topContext.data(), decl );
+
+    if (decl->isExplicitlyDeleted())
+      continue;
+
+    ViableFunction viable( m_topContext.data(), decl, m_constness );
     viable.matchParameters( mergedParams, partial );
 
     viableFunctions << viable;
@@ -274,7 +280,7 @@ ViableFunction OverloadResolver::resolveListViable( const ParameterList& params,
     if ( !decl )
       continue;
     
-    ViableFunction viable( m_topContext.data(), decl );
+    ViableFunction viable( m_topContext.data(), decl, m_constness );
     viable.matchParameters( mergedParams, partial );
 
     if ( viable.isBetter( bestViableFunction ) )
@@ -308,7 +314,7 @@ Declaration* OverloadResolver::applyImplicitTemplateParameters( const ParameterL
   }
 
   const IndexedType* arguments( functionType->indexedArguments() );
-  if ( params.parameters.count() > functionType->indexedArgumentsSize() )
+  if ( static_cast<uint>(params.parameters.count()) > functionType->indexedArgumentsSize() )
     return declaration;
 
   //templateContext contains the template-parameters that we need to find instantiations for
@@ -335,6 +341,7 @@ Declaration* OverloadResolver::applyImplicitTemplateParameters( const ParameterL
     if ( !( *it ) )
     {
       allInstantiated = false;
+      break;
     }
 
   if ( allInstantiated )
@@ -407,6 +414,12 @@ uint OverloadResolver::matchParameterTypes( const AbstractType::Ptr& argumentTyp
   if ( argumentPointer && parameterPointer && (( argumentPointer->modifiers() & AbstractType::ConstModifier ) == ( parameterPointer->modifiers() & AbstractType::ConstModifier ) ) )
     return incrementIfSuccessful( matchParameterTypes( argumentPointer->baseType(), parameterPointer->baseType(), instantiatedTypes, keepValue ) );
 
+  ///In case of arrays on both sides, match the target-types
+    ArrayType::Ptr argumentArray = argumentType.cast<ArrayType>();
+    ArrayType::Ptr parameterArray = parameterType.cast<ArrayType>();
+    if ( argumentArray && parameterArray && (( argumentArray->modifiers() & AbstractType::ConstModifier ) == ( parameterArray->modifiers() & AbstractType::ConstModifier ) ) ) {
+      return incrementIfSuccessful( matchParameterTypes( argumentArray->elementType(), parameterArray->elementType(), instantiatedTypes, keepValue ) );
+    }
 
   if ( CppTemplateParameterType::Ptr templateParam = parameterType.cast<CppTemplateParameterType>() )
   {
@@ -439,10 +452,14 @@ uint OverloadResolver::matchParameterTypes( const AbstractType::Ptr& argumentTyp
 
   if ( identifiedArgument && identifiedParameter )
   {
-    TemplateDeclaration* argumentTemplateDeclaration = dynamic_cast<TemplateDeclaration*>( identifiedArgument->declaration( m_topContext.data() ) );
-    TemplateDeclaration* parameterTemplateDeclaration = dynamic_cast<TemplateDeclaration*>( identifiedParameter->declaration( m_topContext.data() ) );
+    Declaration* argumentDeclaration = identifiedArgument->declaration( m_topContext.data() );
+    Declaration* parameterDeclaration = identifiedParameter->declaration( m_topContext.data() );
+    TemplateDeclaration* argumentTemplateDeclaration = dynamic_cast<TemplateDeclaration*>( argumentDeclaration );
+    TemplateDeclaration* parameterTemplateDeclaration = dynamic_cast<TemplateDeclaration*>( parameterDeclaration );
     if ( !argumentTemplateDeclaration || !parameterTemplateDeclaration )
-      return 1;
+    {
+      return (int)(argumentDeclaration == parameterDeclaration);
+    }
 
     if ( argumentTemplateDeclaration->instantiatedFrom() == parameterTemplateDeclaration->instantiatedFrom() && argumentTemplateDeclaration->instantiatedFrom() )
     {
@@ -641,7 +658,7 @@ uint OverloadResolver::matchParameterTypes( AbstractType::Ptr argumentType, cons
     if ( argumentTemplateDeclarationContext )
     {
       int matchLength = argumentTemplateDeclarationContext->localDeclarations().count();
-      if ( parameterType.templateIdentifiersCount() < matchLength )
+      if ( parameterType.templateIdentifiersCount() < static_cast<uint>(matchLength) )
         matchLength = parameterType.templateIdentifiersCount();
 
       for ( int a = 0; a < matchLength; ++a )
