@@ -20,7 +20,7 @@
 
 #include "templatedeclaration.h"
 
-#include <qatomic.h>
+#include <QThreadStorage>
 #include <kglobal.h>
 
 #include <language/duchain/declaration.h>
@@ -37,6 +37,7 @@
 #include "qtfunctiondeclaration.h"
 #include "cppducontext.h"
 #include "expressionparser.h"
+#include "templateresolver.h"
 #include <language/duchain/classdeclaration.h>
 #include <language/duchain/duchainregister.h>
 #include <util/pushvalue.h>
@@ -74,19 +75,6 @@ namespace Cpp {
   DEFINE_LIST_MEMBER_HASH(SpecialTemplateDeclarationData, m_specializedWith, IndexedType)
 }
 
-struct AtomicIncrementer {
-  typedef QBasicAtomicInt Int;
-  AtomicIncrementer( Int *cnt ) : c(cnt) {
-    c->ref();
-  }
-
-  ~AtomicIncrementer() {
-    c->deref();
-  }
-
-  Int* c;
-};
-
 AbstractType::Ptr applyPointerReference( AbstractType::Ptr ptr, const KDevelop::IndexedTypeIdentifier& id ) {
   AbstractType::Ptr ret = ptr;
 
@@ -157,21 +145,83 @@ TypePtr<DelayedType> containsDelayedType(AbstractType::Ptr type)
 }
 }
 
-///Replaces any DelayedType's in interesting positions with their resolved versions, if they can be resolved.
-struct DelayedTypeResolver : public KDevelop::TypeExchanger {
-  const KDevelop::DUContext* searchContext;
+/**
+ * Thread-local data to ensure recursion limits are not exceeded.
+ * Also holds the implementation data to support defaulte template parameters.
+ */
+struct ThreadLocalData {
+  ThreadLocalData()
+  : delayedDepth(0)
+  , aliasDepth(0)
+  {}
+  // used to apply default template parameters
+  QMultiHash<IndexedQualifiedIdentifier, IndexedType> typeOverloads;
+  // recursion counter for delayed type resolution
+  uint delayedDepth;
+  // recursion counter for alias type resolution
+  uint aliasDepth;
+  uint specializedFromDepth;
+};
 
+#if (QT_VERSION >= 0x040801)
+QThreadStorage<ThreadLocalData> threadData;
+inline ThreadLocalData& threadDataLocal() {
+  return threadData.localData();
+}
+#else
+QThreadStorage<ThreadLocalData*> threadData;
+inline ThreadLocalData& threadDataLocal() {
+  if(!threadData.localData())
+    threadData.setLocalData(new ThreadLocalData());
+  return *threadData.localData();
+}
+#endif
+
+/**
+ * RAII class to push/pop a type overload for a given identifier.
+ */
+struct PushTypeOverload
+{
+  PushTypeOverload(const IndexedQualifiedIdentifier& qid_, const IndexedType& type_)
+  : qid(qid_)
+  , type(type_)
+  , data(threadDataLocal())
+  {
+    data.typeOverloads.insert(qid, type);
+  }
+  ~PushTypeOverload()
+  {
+    data.typeOverloads.remove(qid, type);
+  }
+private:
+  IndexedQualifiedIdentifier qid;
+  IndexedType type;
+  ThreadLocalData& data;
+};
+
+/**
+ * Replaces any DelayedTypes in interesting positions with their resolved versions,
+ * if they can be resolved.
+ */
+struct DelayedTypeResolver : public KDevelop::TypeExchanger
+{
+  const KDevelop::DUContext* searchContext;
   const KDevelop::TopDUContext* source;
-  static AtomicIncrementer::Int depth_counter;
   KDevelop::DUContext::SearchFlags searchFlags;
 
-  DelayedTypeResolver(const KDevelop::DUContext* _searchContext, const KDevelop::TopDUContext* _source, KDevelop::DUContext::SearchFlags _searchFlags = KDevelop::DUContext::NoUndefinedTemplateParams) : searchContext(_searchContext), source(_source), searchFlags(_searchFlags) {
-  }
+  DelayedTypeResolver(const DUContext* _searchContext,
+                      const TopDUContext* _source,
+                      DUContext::SearchFlags _searchFlags = DUContext::NoUndefinedTemplateParams)
+  : searchContext(_searchContext)
+  , source(_source)
+  , searchFlags(_searchFlags)
+  { }
 
   virtual AbstractType::Ptr exchange( const AbstractType::Ptr& type )
   {
-    AtomicIncrementer inc(&depth_counter);
-    if( depth_counter > 30 ) {
+    ThreadLocalData& data = threadDataLocal();
+    PushValue<uint> inc(data.delayedDepth, data.delayedDepth + 1);
+    if( data.delayedDepth > 30 ) {
       kDebug(9007) << "Too much depth in DelayedTypeResolver::exchange, while exchanging" << (type ? type->toString() : QString("(null)"));
       return type;
     }
@@ -180,15 +230,23 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger {
     if( delayedType && delayedType->kind() == DelayedType::Delayed ) {
       QualifiedIdentifier qid = delayedType->identifier().identifier().identifier();
       if( !qid.isExpression() ) {
-        DUContext::SearchItem::PtrList identifiers;
-        identifiers << DUContext::SearchItem::Ptr( new DUContext::SearchItem(qid) );
-        DUContext::DeclarationList decls;
-        if( !searchContext->findDeclarationsInternal( identifiers, searchContext->range().end, AbstractType::Ptr(), decls, source, searchFlags, 0 ) )
-          return type;
-        
+        // look for default template parameters
+        IndexedType indexedType = data.typeOverloads.value(qid);
+        if(!indexedType) {
+          // fall back to normal DUContext search
+          DUContext::SearchItem::PtrList identifiers;
+          identifiers << DUContext::SearchItem::Ptr( new DUContext::SearchItem(qid) );
+          DUContext::DeclarationList decls;
+          if( !searchContext->findDeclarationsInternal( identifiers, searchContext->range().end, AbstractType::Ptr(), decls, source, searchFlags, 0 ) )
+            return type;
 
-        if( !decls.isEmpty() ) {
-          return applyPointerReference(decls[0]->abstractType(), delayedType->identifier());
+          if( !decls.isEmpty() ) {
+            indexedType = decls[0]->indexedType();
+          }
+        }
+
+        if( indexedType.isValid() ) {
+          return applyPointerReference(indexedType.abstractType(), delayedType->identifier());
         }
       }
       ///Resolution as type has failed, or is not appropriate.
@@ -216,7 +274,6 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger {
         if( aType )
           aType->exchangeTypes(this);
 
-        
         return typeCopy;
       }
     }
@@ -231,9 +288,6 @@ struct DelayedTypeResolver : public KDevelop::TypeExchanger {
     AbstractType::Ptr keepAlive;
 };
 
-AtomicIncrementer::Int DelayedTypeResolver::depth_counter = Q_BASIC_ATOMIC_INITIALIZER(0);
-AtomicIncrementer::Int alias_depth_counter = Q_BASIC_ATOMIC_INITIALIZER(0);
-
 // bool operator==( const ExpressionEvaluationResult& left, const ExpressionEvaluationResult& right ) {
 //  return left.type == right.type && left.isInstance == right.isInstance;
 // }
@@ -256,7 +310,8 @@ TemplateDeclaration::TemplateDeclaration()
 {
 }
 
-Declaration* TemplateDeclaration::specialize(IndexedInstantiationInformation specialization, const TopDUContext* topContext, int upDistance) {
+Declaration* TemplateDeclaration::specialize(const IndexedInstantiationInformation& specialization,
+                                             const TopDUContext* topContext, int upDistance) {
   if(!specialization.isValid())
     return dynamic_cast<Declaration*>(this);
   else {
@@ -320,7 +375,30 @@ TemplateDeclaration* TemplateDeclaration::instantiatedFrom() const {
 }
 
 void TemplateDeclaration::setSpecializedFrom(TemplateDeclaration* other) {
-  
+  ThreadLocalData& data = threadDataLocal();
+  PushValue<uint> safety(data.specializedFromDepth, data.specializedFromDepth+1);
+  if(data.specializedFromDepth > 30) {
+    QString withVal = "other";
+    if( other ) {
+      if( other->instantiatedFrom() ) {
+        withVal += ".instantiatedFrom() == " + other->instantiatedFrom()->id(true).qualifiedIdentifier().toString();
+      } else if( other->specializedFrom().data() ) {
+        withVal += ".specialiedFrom().data() == " + other->specializedFrom().data()->identifier().toString();
+      } else {
+        withVal += ".specializedFrom().data() == null && other.instantiatedFrom() == null";
+      }
+    } else {
+      withVal = " == null";
+    }
+    kWarning()
+        << "depth-limit reached while setting specializedFrom"
+        << (specializedFrom().data()
+            ? specializedFrom().data()->identifier().toString()
+            : "this specializedFrom is null")
+        << "with" << withVal;
+    return;
+  }
+
   if(other && other->instantiatedFrom()) {
     setSpecializedFrom(other->instantiatedFrom());
     return;
@@ -434,38 +512,6 @@ bool isTemplateDeclaration(const KDevelop::Declaration* decl) {
   return (bool)dynamic_cast<const TemplateDeclaration*>(decl);
 }
 
-///Applies the default-parameters from basicDeclaration. This must be called AFTER basicDeclaration was instantiated, so the
-///template-parameters are already replaced with their instantiated types
-// InstantiationInformation applyDefaultParameters( const InstantiationInformation& information, Declaration* basicDeclaration )
-// {
-//   DUContext* templateCtx = getTemplateContext(basicDeclaration);
-//   if( !templateCtx )
-//     return information;
-//
-//   if( information.templateParametersSize() >= templateCtx->localDeclarations().count() )
-//     return information;
-//
-//   InstantiationInformation ret(information);
-//
-//   for( int a = information.templateParameters.count(); a < templateCtx->localDeclarations().count(); a++ ) {
-//     TemplateParameterDeclaration* decl = dynamic_cast<TemplateParameterDeclaration*>(templateCtx->localDeclarations()[a]);
-//     if( decl ) {
-//       if( !decl->defaultParameter().isEmpty() ) {
-//         ExpressionEvaluationResult res;
-//         res.type = decl->abstractType();
-//
-//         ret.templateParametersList.append(res.indexed());
-//       }else{
-//         //kDebug(9007) << "missing needed default template-parameter";
-//       }
-//     } else {
-//       kDebug(9007) << "Warning: non template-parameter in template context";
-//     }
-//   }
-//
-//   return ret;
-// }
-
 ///@todo prevent endless recursion when resolving base-classes!(Parent is not yet in du-chain, so a base-class that references it will cause endless recursion)
 CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::DUContext* parentContext, const TopDUContext* source, KDevelop::DUContext* context, const InstantiationInformation& templateArguments, Declaration* instantiatedDeclaration, Declaration* instantiatedFrom, bool doNotRegister )
 {
@@ -609,8 +655,9 @@ CppDUContext<KDevelop::DUContext>* instantiateDeclarationAndContext( KDevelop::D
   if( instantiatedDeclaration && instantiatedDeclaration->abstractType() ) {
         ///an AliasDeclaration represents a C++ "using bla::bla;" declaration.
         if(AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(instantiatedDeclaration)) {
-          AtomicIncrementer safety(&alias_depth_counter);
-          if(alias_depth_counter > 30) {
+          ThreadLocalData& data = threadDataLocal();
+          PushValue<uint> safety(data.aliasDepth, data.delayedDepth + 1);
+          if(data.aliasDepth > 30) {
             kWarning() << "depth-limit reached while resolving alias-declaration" << alias->identifier().toString() << "within" << parentContext->scopeIdentifier(true).toString();
           }else {
             ///For alias declaration, we resolve the declaration that is aliased instead of a type.
@@ -708,8 +755,7 @@ void TemplateDeclaration::deleteAllInstantiations()
 {
   if(m_instantiations.isEmpty() && m_defaultParameterInstantiations.isEmpty())
     return;
-  ENSURE_CHAIN_WRITE_LOCKED
-  
+
   InstantiationsHash instantiations;
   {
     QMutexLocker l(&instantiationsMutex);
@@ -723,104 +769,193 @@ void TemplateDeclaration::deleteAllInstantiations()
     decl->m_instantiatedFrom = 0;
     Declaration* realDecl = dynamic_cast<Declaration*>(decl);
     //Only delete real insantiations, not specializations
-    if(realDecl->isAnonymous()) {
+    //FIXME: before this checked for decl->isAnonymous
+    //This was a problem because some instantiations are not anonymous, so they end up orphaned from their m_instantiatedFrom
+    //If strange crashes start cropping up in template code, this needs more thought
+    if(!decl->specializedFrom().isValid()) {
       Declaration* realDecl = dynamic_cast<Declaration*>(decl);
-      delete realDecl; //It may as well be just a specialization, then we should keep it
+      delete realDecl;
     }
   }
 }
 
-// #define ifDebugMatching(x) x
-#define ifDebugMatching(x)
+//TODO: QHash?
+typedef QMap<IndexedString, AbstractType::Ptr> TemplateParams;
 
-QPair<unsigned int, TemplateDeclaration*> TemplateDeclaration::matchTemplateParameters(Cpp::InstantiationInformation info, const TopDUContext* source) {
-  Q_ASSERT(source);
-  ///@todo match higher scopes
-  InstantiationInformation specializedWith(this->specializedWith().information());
-  
-  Declaration* thisDecl = dynamic_cast<Declaration*>(this);
-   ifDebugMatching( kDebug() << "matching to" << thisDecl->toString() << "parameters:" << info.toString(); )
-
-  if(info.templateParametersSize() != specializedWith.templateParametersSize()) {
-     ifDebugMatching( kDebug() << "template-parameter count doesn't match" << info.toString() << specializedWith.toString(); )
-    return qMakePair(0u, (TemplateDeclaration*)0);
-  }
-  
-  QMap<IndexedString, AbstractType::Ptr> instantiatedParameters;
-  DUContext* templateContext = getTemplateContext(thisDecl, source);
-  if(!templateContext) {
-    return qMakePair(0u, (TemplateDeclaration*)0);
-  }
-  
-  ///Match the arguments, by assigning types to template paremeters
-  foreach(Declaration* parameterDecl, templateContext->localDeclarations()) {
-    instantiatedParameters[parameterDecl->identifier().identifier()] = AbstractType::Ptr();
-     ifDebugMatching( kDebug() << "need value for parameter" << parameterDecl->identifier().identifier().str(); )
-  }
-
-  OverloadResolver resolver(DUContextPointer(thisDecl->context()), TopDUContextPointer(const_cast<TopDUContext*>(source)));
-  
-  uint matchDepth = 1;
-  
+uint matchInstantiationParameters(const InstantiationInformation &info, const InstantiationInformation &matchAgainst,
+                                  const TopDUContext *topCtxt, TemplateParams &requiredParams)
+{
+  TemplateResolver resolver(topCtxt);
+  uint matchQuality = 1;
   for(uint a = 0; a < info.templateParametersSize(); ++a) {
-    uint localMatchDepth = resolver.matchParameterTypes(info.templateParameters()[a].abstractType(), specializedWith.templateParameters()[a].abstractType(), instantiatedParameters, true);
-    if(!localMatchDepth) {
-      ifDebugMatching( kDebug() << "mismatch in parameter" << a; )
-      return qMakePair(0u, (TemplateDeclaration*)0);
-    }
-    matchDepth += localMatchDepth;
+    uint parameterMatchQuality = resolver.matchTemplateParameterTypes(info.templateParameters()[a].abstractType(),
+                                                                      matchAgainst.templateParameters()[a].abstractType(),
+                                                                      requiredParams);
+    if (!parameterMatchQuality)
+      return 0;
+    matchQuality += parameterMatchQuality;
   }
-  
-  for(QMap<IndexedString, AbstractType::Ptr>::iterator it = instantiatedParameters.begin(); it != instantiatedParameters.end(); ++it) {
-    ifDebugMatching( kDebug() << "value for parameter" << it.key().str() << (it.value() ? it.value()->toString() : QString("no type")); )
-    AbstractType::Ptr param(it.value());
-    if(param.isNull()) { //All parameters should have a type assigned
-      return qMakePair(0u, (TemplateDeclaration*)0);
-    }
+  return matchQuality;
+}
+
+//Returns the first (and barring bugs the only) imported template context.
+//A template decl imports one template context, which in turn imports the template context for the next scope, etc.
+DUContext* nextTemplateContext(const DUContext* importingContext, const TopDUContext *topCtxt)
+{
+  foreach( const DUContext::Import &import, importingContext->importedParentContexts() ) {
+    DUContext* c = import.context(topCtxt);
+    if (c && c->type() == DUContext::Template)
+      return c;
   }
-    
-  Cpp::InstantiationInformation instantiateWith(info);
-  instantiateWith.templateParametersList().clear();
+  return 0;
+}
 
-  foreach( Declaration* decl, templateContext->localDeclarations() ) {
-    AbstractType::Ptr type;
-
-    CppTemplateParameterType::Ptr paramType = decl->abstractType().cast<CppTemplateParameterType>();
-    if( paramType ) //Take the type we have assigned.
-      type = instantiatedParameters[decl->identifier().identifier()];
-    else
-      type = decl->abstractType(); //Take the type that was available already earlier
-
-    instantiateWith.addTemplateParameter(type);
-  }
-  
-  ifDebugMatching( kDebug() << "instantiating locally with" << instantiateWith.toString(); )
-
-  TemplateDeclaration* instantiated = this;
-  if(templateContext->localDeclarations().count() != 0)
-    instantiated = dynamic_cast<TemplateDeclaration*>(instantiate( instantiateWith, source, true ));
-  
-  if(!instantiated)
-    return qMakePair(0u, (TemplateDeclaration*)0);
-  
-  ifDebugMatching( kDebug() << "instantiated:" << dynamic_cast<Declaration*>(instantiated)->toString(); )
-  
-  InstantiationInformation instantiationSpecializedWith(instantiated->specializedWith().information());
-  if(instantiationSpecializedWith.templateParametersSize()) {
-    if(instantiationSpecializedWith.templateParametersSize() != info.templateParametersSize()) {
-      ifDebugMatching( kDebug() << "template-parameter size does not match while matching" << thisDecl->toString(); )
-      return qMakePair(0u, (TemplateDeclaration*)0);
+uint TemplateDeclaration::matchInstantiation(IndexedInstantiationInformation indexedInfo, const TopDUContext* topCtxt,
+                                             InstantiationInformation &instantiateWith, bool &instantiationRequired) const
+{
+  DUContext *templateContext = this->templateParameterContext();
+  IndexedInstantiationInformation indexedSpecializedWith = this->specializedWith();
+  uint matchQuality = 1;
+  instantiationRequired = false;
+  while(indexedInfo.isValid() && templateContext)
+  {
+    if (templateContext->localDeclarations().size())
+      instantiationRequired = true;
+    InstantiationInformation info = indexedInfo.information();
+    InstantiationInformation specializedWith = indexedSpecializedWith.information();
+    if (info.templateParametersSize() != specializedWith.templateParametersSize())
+        return 0;
+    if (!info.templateParametersSize())
+    {
+      indexedInfo = info.previousInstantiationInformation;
+      indexedSpecializedWith = specializedWith.previousInstantiationInformation;
+      continue;
     }
-    //Use the information from the specialization-information to build the template-identifiers
-    for(uint a = 0; a < instantiationSpecializedWith.templateParametersSize(); ++a) {
-      if(instantiationSpecializedWith.templateParameters()[a] != info.templateParameters()[a]) {
-        ifDebugMatching( kDebug() << "mismatch in final parameters at" << a << instantiationSpecializedWith.templateParameters()[a].abstractType()->toString() << info.templateParameters()[a].abstractType()->toString(); )
-        return qMakePair(0u, (TemplateDeclaration*)0);
+    TemplateParams requiredParameters;
+    foreach(Declaration* parameterDecl, templateContext->localDeclarations())
+      requiredParameters[parameterDecl->identifier().identifier()] = AbstractType::Ptr();
+    uint match = matchInstantiationParameters(info, specializedWith, topCtxt, requiredParameters);
+    if (!match)
+      return 0;
+    matchQuality += match;
+
+    InstantiationInformation currentInstantiation;
+    foreach( Declaration* decl, templateContext->localDeclarations() )
+    {
+      if( decl->abstractType().cast<CppTemplateParameterType>() )
+      {
+        IndexedString identifier = decl->identifier().identifier();
+        if (requiredParameters[identifier])
+          currentInstantiation.addTemplateParameter(requiredParameters[identifier]); //Take the type we have assigned.
+        else
+          return 0;
+      }
+      else
+        currentInstantiation.addTemplateParameter(decl->abstractType()); //Use the specialized type
+    }
+    currentInstantiation.previousInstantiationInformation = instantiateWith.indexed();
+    instantiateWith = currentInstantiation;
+
+    indexedSpecializedWith = specializedWith.previousInstantiationInformation;
+    indexedInfo = info.previousInstantiationInformation;
+    templateContext = nextTemplateContext(templateContext, topCtxt);
+  }
+  return matchQuality;
+}
+
+
+TemplateDeclaration* TemplateDeclaration::instantiateSpecialization(const InstantiationInformation& info, const TopDUContext* source)
+{
+  TemplateDeclaration *specialization = 0;
+  InstantiationInformation specializationInstantiationInfo;
+  bool instantiationRequired;
+  uint matchQuality = 0;
+  FOREACH_FUNCTION(const IndexedDeclaration& decl, specializations) {
+    //We only use visible specializations here
+    if(source->recursiveImportIndices().contains(decl.indexedTopContext())) {
+      TemplateDeclaration* curSpecialization = dynamic_cast<TemplateDeclaration*>(decl.data());
+      if(curSpecialization) {
+        InstantiationInformation possibleInstantiation;
+        uint match = curSpecialization->matchInstantiation(info.indexed(), source, possibleInstantiation, instantiationRequired);
+        if(match > matchQuality) {
+          matchQuality = match;
+          specializationInstantiationInfo = possibleInstantiation;
+          specialization = curSpecialization;
+        }
       }
     }
   }
+
+  if (specialization) {
+    if (!instantiationRequired)
+      return specialization;
+    else
+      return dynamic_cast<TemplateDeclaration*>(specialization->instantiate(specializationInstantiationInfo, source, true));
+  }
   
-  return qMakePair((unsigned int)(info.templateParametersSize() - templateContext->localDeclarations().count())*1000 + matchDepth, instantiated);
+  return 0;
+}
+
+void applyDefaultParameters(const DUContext* templateContext, const TopDUContext* source,
+                            const DUContext* surroundingContext,
+                            InstantiationInformation* templateArguments)
+{
+  Q_ASSERT(templateContext);
+  Q_ASSERT(templateContext->type() == DUContext::Template);
+  Q_ASSERT(source);
+  Q_ASSERT(surroundingContext);
+
+  const int totalParameters = templateContext->localDeclarations().count();
+  KDevVarLengthArray<IndexedType, 10> explicitParameters = templateArguments->templateParametersList();
+
+  if(totalParameters <= explicitParameters.size()
+     //TODO: why is this required?
+     && (explicitParameters.size() != 1 || explicitParameters.at(0).isValid()))
+  {
+    // nothing to do
+    return;
+  }
+
+  KDevVarLengthArray<IndexedType, 10> appliedParameters;
+  int currentArgument = 0;
+
+  QVector<PushTypeOverload*> typeOverloads;
+  foreach(Declaration* decl, templateContext->localDeclarations()) {
+    TemplateParameterDeclaration* templateDecl = dynamic_cast<TemplateParameterDeclaration*>(decl);
+    Q_ASSERT(templateDecl); //Only template-parameter declarations are allowed in template-contexts
+
+    IndexedType type = decl->indexedType();
+    Q_ASSERT(type.isValid());
+    if( currentArgument < explicitParameters.size()
+        && explicitParameters.at(currentArgument).isValid() )
+    {
+      // use explicit parameter
+      type = explicitParameters.at(currentArgument);
+      Q_ASSERT(type);
+    } else if(templateDecl->hasDefaultParameter()) {
+      // Apply default-parameter
+      Q_ASSERT(!templateDecl->defaultParameter().isEmpty());
+      DelayedType::Ptr delayed( new DelayedType() );
+      delayed->setIdentifier( IndexedTypeIdentifier(templateDecl->defaultParameter()) );
+      type = resolveDelayedTypes( delayed.cast<AbstractType>(), surroundingContext, source)->indexed();
+    } // else the parameter is missing
+
+    //TODO: why is this neccessary?
+    if(type.abstractType().cast<CppTemplateParameterType>()) {
+      ++currentArgument;
+      continue;
+    }
+
+    appliedParameters << type;
+    if(type != decl->indexedType()) {
+      // add type overload
+      typeOverloads << new PushTypeOverload(decl->qualifiedIdentifier(), type);
+    }
+    ++currentArgument;
+  }
+
+  qDeleteAll(typeOverloads);
+  templateArguments->templateParametersList() = appliedParameters;
 }
 
 Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _templateArguments, const TopDUContext* source, bool forceLocal )
@@ -832,6 +967,9 @@ Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _
   if( m_instantiatedFrom && !forceLocal)
     return m_instantiatedFrom->instantiate( templateArguments, source );
 
+  if ( specializedFrom().data() && !forceLocal )
+    return dynamic_cast<TemplateDeclaration*>(specializedFrom().declaration())->instantiate(templateArguments, source);
+
   {
     QMutexLocker l(&instantiationsMutex);
     {
@@ -881,23 +1019,8 @@ Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _
 //   kDebug() << decl->qualifiedIdentifier().toString() << "got template-context" << templateContext << templateArguments.toString();
 
   if(!forceLocal) {
-    ///Apply default-parameters
-    if(templateContext && (static_cast<uint>(templateContext->localDeclarations().count()) > templateArguments.templateParametersSize() || (templateArguments.templateParametersSize() == 1 && !templateArguments.templateParameters()[0].isValid() && templateContext->localDeclarations().count() >= 1))) {
-      //Do a little fake instantiation so all the default-parameters are correctly applied right here. Then, they can be used
-      //during specialization-matching.
-      DUContext* newTemplateContext = instantiateDeclarationAndContext( surroundingContext, source, templateContext, templateArguments, 0, 0, true );
-      
-      templateArguments.templateParametersList().clear();
-  //     kDebug() << "count in copied:" << newTemplateContext->localDeclarations().count();
-      foreach(Declaration* decl, newTemplateContext->localDeclarations())
-        if(!decl->type<CppTemplateParameterType>()) {
-  //         kDebug() << "inserting" << decl->abstractType()->toString();
-          templateArguments.addTemplateParameter(decl->abstractType());
-        }else{
-  //         kDebug() << "not inserting";
-        }
-      
-      delete newTemplateContext;
+    if(templateContext) {
+      applyDefaultParameters(templateContext, source, surroundingContext, &templateArguments);
     }
 
     ///Check whether there is type-aliases in the parameters that need to be resolved
@@ -969,24 +1092,8 @@ Declaration* TemplateDeclaration::instantiate( const InstantiationInformation& _
     reserveInstantiation(templateArguments.indexed());
   }
 
-  TemplateDeclaration* instantiatedSpecialization =  0;
-  uint specializationMatchQuality = 0;
-  //Check whether there is a specialization that matches the template parameters
-  FOREACH_FUNCTION(const IndexedDeclaration& decl, specializations) {
-    //We only use visible specializations here
-    
-    if(source->recursiveImportIndices().contains(decl.indexedTopContext())) {
-      TemplateDeclaration* tempDecl = dynamic_cast<TemplateDeclaration*>(decl.data());
-      if(tempDecl) {
-        QPair<uint, TemplateDeclaration*> match = tempDecl->matchTemplateParameters(templateArguments, source);
-        if(match.first > specializationMatchQuality && match.second) {
-          instantiatedSpecialization = match.second;
-          specializationMatchQuality = match.first;
-        }
-      }
-    }
-  }
-  
+  TemplateDeclaration *instantiatedSpecialization = instantiateSpecialization(templateArguments, source);
+
   //We have reserved the instantiation, so it must have stayed untouched
   Q_ASSERT(m_instantiations[templateArguments.indexed()] == 0);
   
@@ -1060,11 +1167,6 @@ AbstractType::Ptr resolveDelayedTypes( AbstractType::Ptr type, const KDevelop::D
   }
 }
 
-// void TemplateDeclaration::setInstantiatedWith(const InstantiationInformation& with) {
-//   Q_ASSERT(!m_instantiatedFrom);
-//   m_instantiatedWith = with.indexed();
-// }
-
 IndexedInstantiationInformation TemplateDeclaration::instantiatedWith() const {
   return m_instantiatedWith;
 }
