@@ -18,8 +18,6 @@
 
 #include "typebuilder.h"
 
-#include <ktexteditor/smartrange.h>
-
 #include <language/duchain/identifier.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/forwarddeclaration.h>
@@ -41,30 +39,44 @@
 #include "debugbuilders.h"
 #include <language/duchain/types/typealiastype.h>
 #include <util/pushvalue.h>
+#include "typeutils.h"
+#include <functional>
 
 using namespace KDevelop;
 using namespace Cpp;
 
-///@todo Make this unneeded in this place
-namespace Cpp {
-bool isTemplateDependent(Declaration* decl);
-}
-
 QString stringFromSessionTokens( ParseSession* session, int start_token, int end_token ) {
     int startPosition = session->token_stream->position(start_token);
     int endPosition = session->token_stream->position(end_token);
     return QString::fromUtf8( stringFromContents(session->contentsVector(), startPosition, endPosition - startPosition) );
 }
 
-TypeBuilder::TypeBuilder()
-  : TypeBuilderBase(), m_inTypedef(false), m_lastTypeWasInstance(false)
+bool isConstexpr(ParseSession* session, const ListNode<uint> *storageSpec)
+{
+  if (storageSpec) {
+    const ListNode<uint> *it = storageSpec->toFront();
+    const ListNode<uint> *end = it;
+    do {
+      int kind = session->token_stream->kind(it->element);
+      if (kind == Token_constexpr)
+        return true;
+
+      it = it->next;
+    } while (it != end);
+  }
+
+  return false;
+}
+
+TypeBuilder::TypeBuilder(ParseSession* session)
+  : ContextBuilder(session), m_inTypedef(false), m_lastTypeWasInstance(false), m_lastTypeWasAuto(false)
 {
 }
 
 void TypeBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 {
   if(m_onlyComputeSimplified) {
-    TypeBuilderBase::visitClassSpecifier(node);
+    ContextBuilder::visitClassSpecifier(node);
     return;
   }
   PushValue<bool> setNotInTypedef(m_inTypedef, false);
@@ -76,7 +88,7 @@ void TypeBuilder::visitClassSpecifier(ClassSpecifierAST *node)
 
   classTypeOpened( currentAbstractType() ); //This callback is needed, because the type of the class-declaration needs to be set early so the class can be referenced from within itself
 
-  TypeBuilderBase::visitClassSpecifier(node);
+  ContextBuilder::visitClassSpecifier(node);
 
   closeType();
 }
@@ -101,13 +113,13 @@ void TypeBuilder::visitBaseSpecifier(BaseSpecifierAST *node)
     }
   }
 
-  TypeBuilderBase::visitBaseSpecifier(node);
+  ContextBuilder::visitBaseSpecifier(node);
 }
 
 void TypeBuilder::visitEnumSpecifier(EnumSpecifierAST *node)
 {
   if(m_onlyComputeSimplified) {
-    TypeBuilderBase::visitEnumSpecifier(node);
+    ContextBuilder::visitEnumSpecifier(node);
     return;
   }
   
@@ -115,7 +127,7 @@ void TypeBuilder::visitEnumSpecifier(EnumSpecifierAST *node)
 
   openType(EnumerationType::Ptr(new EnumerationType()));
 
-  TypeBuilderBase::visitEnumSpecifier(node);
+  ContextBuilder::visitEnumSpecifier(node);
 
   closeType();
 }
@@ -123,7 +135,7 @@ void TypeBuilder::visitEnumSpecifier(EnumSpecifierAST *node)
 void TypeBuilder::visitEnumerator(EnumeratorAST* node)
 {
   if(m_onlyComputeSimplified) {
-    TypeBuilderBase::visitEnumerator(node);
+    ContextBuilder::visitEnumerator(node);
     return;
   }
   
@@ -186,7 +198,7 @@ void TypeBuilder::visitEnumerator(EnumeratorAST* node)
     enumerator->setValue<qint64>(m_currentEnumeratorValue);
   }
 
-  TypeBuilderBase::visitEnumerator(node);
+  ContextBuilder::visitEnumerator(node);
 
   closeType();
 
@@ -198,6 +210,21 @@ bool TypeBuilder::lastTypeWasInstance() const
   return m_lastTypeWasInstance;
 }
 
+void TypeBuilder::setLastTypeWasInstance(bool wasInstance)
+{
+  m_lastTypeWasInstance = wasInstance;
+}
+
+bool TypeBuilder::lastTypeWasAuto() const
+{
+  return m_lastTypeWasAuto;
+}
+
+void TypeBuilder::setLastTypeWasAuto(bool wasAuto)
+{
+  m_lastTypeWasAuto = wasAuto;
+}
+
 void TypeBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST *node)
 {
   if(m_onlyComputeSimplified) {
@@ -215,7 +242,7 @@ void TypeBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST *node)
     //For typename, just find the type and return
     bool openedType = openTypeFromName(node->name, parseConstVolatile(editor()->parseSession(), node->cv));
 
-    TypeBuilderBase::visitElaboratedTypeSpecifier(node);
+    ContextBuilder::visitElaboratedTypeSpecifier(node);
 
     if(openedType)
       closeType();
@@ -259,12 +286,13 @@ void TypeBuilder::visitElaboratedTypeSpecifier(ElaboratedTypeSpecifierAST *node)
 
   // TODO.. figure out what to do with this now... parseConstVolatile(node->cv);
 
-  TypeBuilderBase::visitElaboratedTypeSpecifier(node);
+  ContextBuilder::visitElaboratedTypeSpecifier(node);
 
   if (type)
     closeType();
 }
 
+///TODO: share code with TypeASTVisitor
 void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
 {
   if(m_onlyComputeSimplified) {
@@ -273,12 +301,42 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
   
   bool openedType = false;
   m_lastTypeWasInstance = false;
+  m_lastTypeWasAuto = false;
+
+  if ((node->isTypeof || node->isDecltype) && node->expression) {
+    bool isDecltypeInParen = false;
+        if (node->isDecltype && node->expression->kind == AST::Kind_PrimaryExpression) {
+          int startPosition = editor()->parseSession()->token_stream->position(node->expression->start_token);
+          static IndexedString paren("(");
+          isDecltypeInParen = editor()->parseSession()->contentsVector()[startPosition] == paren.index();
+        }
 
-  if (node->type_of && node->expression) {
     node->expression->ducontext = currentContext();
-    ExpressionParser parser;
-    ExpressionEvaluationResult result = parser.evaluateType(node->expression, editor()->parseSession());
-    openType(result.type.abstractType());
+    ExpressionParser parser(false, false, isDecltypeInParen, m_mapAst);
+    ExpressionEvaluationResult result = parser.evaluateType(node->expression, editor()->parseSession(), topContext());
+    AbstractType::Ptr type = result.type.abstractType();
+    // make reference for decltype in additional parens - but only if it's not already a reference
+    // see spec 7.1.6/4
+    if (isDecltypeInParen && type && !TypeUtils::isReferenceType(type))
+    {
+      // type might already be a ref type
+      ReferenceType::Ptr refType = ReferenceType::Ptr(new ReferenceType);
+      refType->setBaseType(type);
+      type = refType.cast<AbstractType>();
+    }
+
+    if(!type)
+    {
+      DUChainReadLocker lock(DUChain::lock());
+      DelayedType::Ptr delayed( new DelayedType() );
+      delayed->setIdentifier( IndexedTypeIdentifier( stringFromSessionTokens(editor()->parseSession(),
+                                                     node->expression->start_token,
+                                                     node->expression->end_token).trimmed(), true ) );
+      delayed->setKind( templateDeclarationDepth() ? DelayedType::Delayed : DelayedType::Unresolved );
+      type = delayed.cast<AbstractType>();
+    }
+
+    openType(type);
     openedType = true;
   } else if (node->integrals) {
     uint type = IntegralType::TypeNone;
@@ -292,6 +350,12 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
         case Token_char:
           type = IntegralType::TypeChar;
           break;
+        case Token_char16_t:
+          type = IntegralType::TypeChar16_t;
+          break;
+        case Token_char32_t:
+          type = IntegralType::TypeChar32_t;
+          break;
         case Token_wchar_t:
           type = IntegralType::TypeWchar_t;
           break;
@@ -325,6 +389,9 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
         case Token_void:
           type = IntegralType::TypeVoid;
           break;
+        case Token_auto:
+          m_lastTypeWasAuto = true;
+          break;
       }
 
       it = it->next;
@@ -344,62 +411,125 @@ void TypeBuilder::visitSimpleTypeSpecifier(SimpleTypeSpecifierAST *node)
     openedType = openTypeFromName(node->name, parseConstVolatile(editor()->parseSession(), node->cv));
   }
 
-  TypeBuilderBase::visitSimpleTypeSpecifier(node);
+  ContextBuilder::visitSimpleTypeSpecifier(node);
 
   if (openedType)
     closeType();
 }
 
+void TypeBuilder::createIntegralTypeForExpression(ExpressionAST* expression)
+{
+  if (!expression) {
+    return;
+  }
+
+  IntegralType::Ptr integral = lastType().cast<IntegralType>();
+  if (!integral && m_lastTypeWasAuto) {
+    ReferenceType::Ptr ref = lastType().cast<ReferenceType>();
+    if (ref) {
+      integral = ref->baseType().cast<IntegralType>();
+    }
+  }
+
+  if (!integral) {
+    return;
+  }
+
+  if (!(integral->modifiers() & AbstractType::ConstModifier) && !m_lastTypeWasAuto) {
+    return;
+  }
+
+  bool openedType = false;
+  bool delay = false;
+  ///@todo This is nearly a copy of visitEnumerator and parts of visitSimpleTypeSpecifier, merge it
+  {
+    //Parse the expression, and create a CppConstantIntegralType, since we know the value
+    Cpp::ExpressionParser parser;
+    DUChainReadLocker lock(DUChain::lock());
+    expression->ducontext = currentContext();
+    Cpp::ExpressionEvaluationResult res = parser.evaluateType( expression, editor()->parseSession() );
+
+    //Delay the type-resolution of template-parameters
+    if( res.allDeclarations.size() ) {
+      Declaration* decl = res.allDeclarations[0].getDeclaration(currentContext()->topContext());
+      ///@todo Do a check on all involved types, also template parameters, by giving the parameter to evaluateType
+      if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl))
+        delay = true;
+    }
+
+    if ( !delay && res.isValid() && res.isInstance ) {
+      const AbstractType::Ptr& type = prepareTypeForExpression(res.type.abstractType(), integral->modifiers());
+      if (type) {
+        openType( type );
+        openedType = true;
+      }
+    }
+  }
+  if( delay || !openedType ) {
+    ///Only record the strings, because these expressions may depend on template-parameters and thus must be evaluated later
+    QString str = stringFromSessionTokens( editor()->parseSession(), expression->start_token, expression->end_token ).trimmed();
+
+    QualifiedIdentifier id( str, true );
+
+    openDelayedType(IndexedTypeIdentifier(id), expression, DelayedType::Delayed);
+    openedType = true;
+  }
+
+  if(openedType)
+    closeType();
+}
+
+AbstractType::Ptr TypeBuilder::prepareTypeForExpression(AbstractType::Ptr type, quint64 modifiers)
+{
+  if (!m_lastTypeWasAuto) {
+    return type;
+  }
+
+  // remove references or aliases
+  type = TypeUtils::realType( type, topContext() );
+  // Turn "5" into "int"
+  type = TypeUtils::removeConstants( type, topContext() );
+
+  if (!type) {
+    // NOTE: the type might not be valid anymore, see https://bugs.kde.org/show_bug.cgi?id=318972
+    return type;
+  }
+
+  // ensure proper const modifier is set
+  type->setModifiers( modifiers );
+
+  if (ReferenceType::Ptr ref = lastType().cast<ReferenceType>()) {
+    ref->setBaseType( type );
+    type = ref.cast<AbstractType>();
+  }
+
+  return type;
+}
+
 void TypeBuilder::createTypeForInitializer(InitializerAST *node) {
   if(m_onlyComputeSimplified) {
     return;
   }
-  
-  IntegralType::Ptr integral = lastType().cast<IntegralType>();
-  if(integral && (integral->modifiers() & AbstractType::ConstModifier) && node->initializer_clause && node->initializer_clause->expression) {
-    //Parse the expression, and create a CppConstantIntegralType, since we know the value
-    Cpp::ExpressionParser parser;
 
-    bool openedType = false;
-    Cpp::ExpressionEvaluationResult res;
-
-    bool delay = false;
-    ///@todo This is nearly a copy of visitEnumerator, merge it
-    if(!delay) {
-      DUChainReadLocker lock(DUChain::lock());
-      node->initializer_clause->expression->ducontext = currentContext();
-      res = parser.evaluateType( node->initializer_clause->expression, editor()->parseSession() );
-
-      //Delay the type-resolution of template-parameters
-      if( res.allDeclarations.size() ) {
-        Declaration* decl = res.allDeclarations[0].getDeclaration(currentContext()->topContext());
-        ///@todo Do a check on all involved types, also template parameters, by giving the parameter to evaluateType
-        if( dynamic_cast<TemplateParameterDeclaration*>(decl) || isTemplateDependent(decl))
-          delay = true;
-      }
-
-      if ( !delay && res.isValid() && res.isInstance ) {
-        openType( res.type.abstractType() );
-        openedType = true;
-      }
-    }
-    if( delay || !openedType ) {
-      QString str;
-      ///Only record the strings, because these expressions may depend on template-parameters and thus must be evaluated later
-      str += stringFromSessionTokens( editor()->parseSession(), node->initializer_clause->expression->start_token, node->initializer_clause->expression->end_token );
-
-      QualifiedIdentifier id( str.trimmed(), true );
-
-      openDelayedType(IndexedTypeIdentifier(id), node, DelayedType::Delayed);
-      openedType = true;
-    }
-    
-    if(openedType)
-      closeType();
+  ExpressionAST* expression = 0;
+  if (node->initializer_clause && node->initializer_clause->expression) {
+    // auto foo = ...;
+    expression = node->initializer_clause->expression;
+  } else {
+    // auto foo(...);
+    expression = node->expression;
   }
+
+  createIntegralTypeForExpression(expression);
 }
 
-void TypeBuilder::closeTypeForInitializer(InitializerAST* /*node*/) {
+void TypeBuilder::createTypeForCondition(ConditionAST* node)
+{
+  if (m_onlyComputeSimplified) {
+    return;
+  }
+
+  createIntegralTypeForExpression(node->expression);
 }
 
 bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass) {
@@ -411,7 +541,7 @@ bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass
   bool delay = false;
 
   if(!delay) {
-    SimpleCursor pos = editor()->findPosition(name->start_token, EditorIntegrator::FrontEdge);
+    CursorInRevision pos = editor()->findPosition(name->start_token, CppEditorIntegrator::FrontEdge);
     DUChainReadLocker lock(DUChain::lock());
     ifDebug( kDebug() << "searching" << id.toString(); )
     ifDebugCurrentFile( kDebug() << "searching" << id.toString(); )
@@ -427,33 +557,40 @@ bool TypeBuilder::openTypeFromName(NameAST* name, uint modifiers, bool needClass
     if(!delay) {
       
       ifDebug( if( dec.count() > 1 ) kDebug(9007) << id.toString() << "was found" << dec.count() << "times"; )
-      
+
+      // see section 3.4.1-7 in the cpp spec on unqualified name lookup
+      // and https://bugs.kde.org/show_bug.cgi?id=273658 for a bug report
+      AbstractType::Ptr type;
+      int matchQuality = 0; // we just pick the size of the qualified identifier as match quality
+      bool wasInstance = false;
       foreach( Declaration* decl, dec ) {
         AbstractType::Ptr unAliased = TypeUtils::unAliasedType(decl->abstractType());
         if( needClass && !unAliased.cast<CppClassType>() )
           continue;
 
-        if (decl->abstractType() ) {
-          if(decl->kind() == KDevelop::Declaration::Instance)
-            m_lastTypeWasInstance = true;
+        if(unAliased.cast<DelayedType>())
+          continue;
 
+        if (decl->abstractType()) {
           //kDebug(9007) << "found for" << id.toString() << ":" << decl->toString() << "type:" << decl->abstractType()->toString() << "context:" << decl->context();
 
-          AbstractType::Ptr type = decl->abstractType();
-          
-          if(unAliased.cast<DelayedType>()) {
-            continue;
+          const int quality = decl->qualifiedIdentifier().count();
+          if (matchQuality < quality) {
+            // better quality, prefer over old match
+            type = decl->abstractType();
+            matchQuality = quality;
+            wasInstance = decl->kind() == KDevelop::Declaration::Instance;
           }
-
-          openedType = true;
-
-          if(type)
-            applyModifiers(type, modifiers);
-
-          openType(type);
-          break;
         }
       }
+
+      if (type) {
+        m_lastTypeWasInstance = wasInstance;
+        applyModifiers(type, modifiers);
+        openType(type);
+
+        openedType = true;
+      }
     }
 
     if(!openedType)
@@ -500,7 +637,7 @@ void TypeBuilder::visitTypedef(TypedefAST* node)
   
 //   openType(KDevelop::TypeAliasType::Ptr(new KDevelop::TypeAliasType()));
 
-  TypeBuilderBase::visitTypedef(node);
+  ContextBuilder::visitTypedef(node);
 
 //   closeType();
 }
@@ -523,10 +660,10 @@ void TypeBuilder::visitFunctionDeclaration(FunctionDefinitionAST* node)
 {
   clearLastType();
 
-  if(!node->init_declarator && node->type_specifier)
+  if(!node->declarator && node->type_specifier)
     m_typeSpecifierWithoutInitDeclarators = node->type_specifier->start_token;
 
-  TypeBuilderBase::visitFunctionDeclaration(node);
+  ContextBuilder::visitFunctionDeclaration(node);
 }
 
 void TypeBuilder::visitSimpleDeclaration(SimpleDeclarationAST* node)
@@ -540,9 +677,12 @@ void TypeBuilder::visitSimpleDeclaration(SimpleDeclarationAST* node)
 
   AbstractType::Ptr baseType = lastType();
 
+  if (baseType && isConstexpr(editor()->parseSession(), node->storage_specifiers)) {
+    baseType->setModifiers(baseType->modifiers() | AbstractType::ConstModifier);
+  }
+
   if (node->init_declarators) {
     const ListNode<InitDeclaratorAST*> *it = node->init_declarators->toFront(), *end = it;
-
     do {
       visit(it->element);
       // Reset last type to be the base type
@@ -565,12 +705,15 @@ void TypeBuilder::visitPtrOperator(PtrOperatorAST* node)
   
   bool typeOpened = false;
   if (node->op) {
-    QString op = editor()->tokenToString(node->op);
+    const QString op = editor()->tokenToString(node->op);
     if (!op.isEmpty()) {
       if (op[0] == '&') {
         ReferenceType::Ptr pointer(new ReferenceType());
         pointer->setModifiers(parseConstVolatile(editor()->parseSession(), node->cv));
         pointer->setBaseType(lastType());
+        if (op.size() == 2 && op[1] == '&')
+          pointer->setIsRValue(true);
+
         openType(pointer);
         typeOpened = true;
 
@@ -584,12 +727,22 @@ void TypeBuilder::visitPtrOperator(PtrOperatorAST* node)
     }
   }
 
-  TypeBuilderBase::visitPtrOperator(node);
+  ContextBuilder::visitPtrOperator(node);
 
   if (typeOpened)
     closeType();
 }
 
+void TypeBuilder::visitPtrToMember(PtrToMemberAST *node)
+{
+  PtrToMemberType::Ptr pointer(new PtrToMemberType);
+  pointer->setBaseType(lastType());
+  ContextBuilder::visitPtrToMember(node);
+  pointer->setClassType(lastType());
+  openType(pointer);
+  closeType();
+}
+
 FunctionType* TypeBuilder::openFunction(DeclaratorAST *node)
 {
   FunctionType* functionType = new FunctionType();
@@ -603,6 +756,15 @@ FunctionType* TypeBuilder::openFunction(DeclaratorAST *node)
   return functionType;
 }
 
+void TypeBuilder::visitTrailingReturnType(TrailingReturnTypeAST* node)
+{
+  DefaultVisitor::visitTrailingReturnType(node);
+
+  FunctionType::Ptr funcType = currentType<FunctionType>();
+  if (lastType() && funcType)
+    funcType->setReturnType(lastType());
+}
+
 void TypeBuilder::createTypeForDeclarator(DeclaratorAST *node) {
   // Custom code - create array types
   if (node->array_dimensions) {
@@ -614,17 +776,18 @@ void TypeBuilder::createTypeForDeclarator(DeclaratorAST *node) {
     } while (it != end);
   }
 
-  if (node->parameter_declaration_clause)
+  if (node->parameter_declaration_clause) {
     // New function type
     openType(FunctionType::Ptr(openFunction(node)));
+  }
 }
 
 void TypeBuilder::closeTypeForDeclarator(DeclaratorAST *node) {
-  if (node->parameter_declaration_clause)
+  if (node->parameter_declaration_clause) {
     closeType();
+  }
 }
 
-
 void TypeBuilder::visitArrayExpression(ExpressionAST* expression)
 {
   if(m_onlyComputeSimplified) {
@@ -700,7 +863,7 @@ void TypeBuilder::visitTemplateParameter(TemplateParameterAST *ast)
 //   if(!ast->parameter_declaration)
     openType(CppTemplateParameterType::Ptr(new CppTemplateParameterType()));
 
-  TypeBuilderBase::visitTemplateParameter(ast);
+  ContextBuilder::visitTemplateParameter(ast);
   
 //   if(!ast->parameter_declaration)
     closeType();
@@ -709,9 +872,9 @@ void TypeBuilder::visitTemplateParameter(TemplateParameterAST *ast)
 
 void TypeBuilder::visitParameterDeclaration(ParameterDeclarationAST* node)
 {
-  TypeBuilderBase::visitParameterDeclaration(node);
+  ContextBuilder::visitParameterDeclaration(node);
 
-  if (hasCurrentType() && !m_onlyComputeSimplified) {
+  if (currentAbstractType() && !m_onlyComputeSimplified) {
     if (FunctionType::Ptr function = currentType<FunctionType>()) {
       function->addArgument(lastType());
     }
@@ -721,7 +884,7 @@ void TypeBuilder::visitParameterDeclaration(ParameterDeclarationAST* node)
 
 void TypeBuilder::visitUsing(UsingAST * node)
 {
-  TypeBuilderBase::visitUsing(node);
+  ContextBuilder::visitUsing(node);
 
   if(!m_onlyComputeSimplified)
   {
@@ -731,3 +894,45 @@ void TypeBuilder::visitUsing(UsingAST * node)
       closeType();
   }
 }
+
+bool TypeBuilder::openTypeFromName(QualifiedIdentifier id, AST* typeNode, bool needClass) {
+    bool openedType = false;
+
+    bool delay = false;
+
+    if (!delay) {
+        CursorInRevision pos(editorFindRange(typeNode, typeNode).start);
+        DUChainReadLocker lock(DUChain::lock());
+
+        QList<Declaration*> dec = searchContext()->findDeclarations(id, pos);
+
+        if ( dec.isEmpty() )
+            delay = true;
+
+        if (!delay) {
+            foreach( Declaration* decl, dec ) {
+                // gcc 4.0.1 doesn't eath this // if( needClass && !decl->abstractType().cast<StructureType>() )
+                if ( needClass && !decl->abstractType().cast(static_cast<StructureType *>(0)) )
+                    continue;
+
+                if (decl->abstractType() ) {
+                    openedType = true;
+                    openType(decl->abstractType());
+                    break;
+                }
+            }
+        }
+
+        if (!openedType)
+            delay = true;
+    }
+
+    /*if(delay) {
+    //Either delay the resolution for template-dependent types, or create an unresolved type that stores the name.
+    openedType = true;
+    openDelayedType(id, name, templateDeclarationDepth() ? DelayedType::Delayed : DelayedType::Unresolved );
+
+    ifDebug( if(templateDeclarationDepth() == 0) kDebug() << "no declaration found for" << id.toString() << "in context \"" << searchContext()->scopeIdentifier(true).toString() << "\"" << "" << searchContext() )
+    }*/
+    return openedType;
+}
