@@ -62,6 +62,11 @@ void TestParser::initTestCase()
   core->initialize(KDevelop::Core::NoUi);
 }
 
+void TestParser::cleanupTestCase()
+{
+  KDevelop::TestCore::shutdown();
+}
+
 void TestParser::testSymbolTable()
 {
   NameTable table;
@@ -404,7 +409,7 @@ void TestParser::testComments4()
 void TestParser::testComments5()
 {
   CommentFormatter formatter;
-  QByteArray method("//TranslationUnitComment\n  //FIXME comment\n //this is TODO\n /* TODO: comment */\n  int i;  // another TODO \n // Just a simple comment\nint j;\n int main(void) {\n // TODO COMMENT\n}\n");
+  QByteArray method("//TranslationUnitComment\n  //FIXME comment\n //this is TODO\n /* TODO: comment */\n  int i;  // another TODO \n // Just a simple comment\nint j;\n int main(void) {\n // TODO COMMENT\n}\n// Non-ascii TODO 例えば\n");
   int initial_size = control.problems().size();  // Remember existing number of problems
   TranslationUnitAST* ast = parse(method);
 
@@ -418,7 +423,7 @@ void TestParser::testComments5()
   QCOMPARE(formatter.formatComment(it->element->comments, lastSession), QByteArray("Just a simple comment"));
 
   QList<KDevelop::ProblemPointer> problem_list = control.problems();
-  QCOMPARE(problem_list.size(), initial_size + 5); // 5 to-dos
+  QCOMPARE(problem_list.size(), initial_size + 6); // 6 to-dos
   KDevelop::ProblemPointer problem = problem_list[initial_size];
   QCOMPARE(problem->description(), QString("FIXME comment"));
   QCOMPARE(problem->source(), KDevelop::ProblemData::ToDo);
@@ -448,6 +453,12 @@ void TestParser::testComments5()
   QCOMPARE(problem->source(), KDevelop::ProblemData::ToDo);
   QCOMPARE(problem->finalLocation().start, KDevelop::SimpleCursor(8, 4));
   QCOMPARE(problem->finalLocation().end, KDevelop::SimpleCursor(8, 16));
+
+  problem = problem_list[initial_size + 5];
+  QCOMPARE(problem->description(), QString::fromUtf8("Non-ascii TODO 例えば"));
+  QCOMPARE(problem->source(), KDevelop::ProblemData::ToDo);
+  QCOMPARE(problem->finalLocation().start, KDevelop::SimpleCursor(10, 3));
+  QCOMPARE(problem->finalLocation().end, KDevelop::SimpleCursor(10, 27));
 }
 
 void TestParser::testComments6() {
@@ -485,11 +496,24 @@ void TestParser::testPreprocessor() {
   QCOMPARE(preprocess("#if 1l\n #define N 10\n#else\n#define N 20\n#endif\nN\n").trimmed(), QString("10"));
   QCOMPARE(preprocess("#if 1lu\n #define N 10\n#else\n#define N 20\n#endif\nN\n").trimmed(), QString("10"));
   QCOMPARE(preprocess("#if 1ul\n #define N 10\n#else\n#define N 20\n#endif\nN\n").trimmed(), QString("10"));
-  QEXPECT_FAIL("", "not yet handled", Continue);
   QCOMPARE(preprocess("#if 1ll\n #define N 10\n#else\n#define N 20\n#endif\nN\n").trimmed(), QString("10"));
-  QEXPECT_FAIL("", "not yet handled", Continue);
   QCOMPARE(preprocess("#if 1llu\n #define N 10\n#else\n#define N 20\n#endif\nN\n").trimmed(), QString("10"));
 
+  QCOMPARE(preprocess("#if ~0ull == 0u + ~0u\n 10\n #endif\n").trimmed(), QString("10"));
+
+  QCOMPARE(preprocess("#if 1Ul\n 10\n #endif\n").trimmed(), QString("10"));
+  QCOMPARE(preprocess("#if 1Lu\n 10\n #endif\n").trimmed(), QString("10"));
+  QCOMPARE(preprocess("#if 1LlU\n 10\n #endif\n").trimmed(), QString("10"));
+
+  QCOMPARE(preprocess("#if 1_u\n #endif\n").trimmed(), QString("*ERROR*"));
+  QCOMPARE(preprocess("#if 1u_\n #endif\n").trimmed(), QString("*ERROR*"));
+
+  QCOMPARE(preprocess("#if 1uu\n #endif\n").trimmed(), QString("*ERROR*"));
+  QCOMPARE(preprocess("#if 1lll\n #endif\n").trimmed(), QString("*ERROR*"));
+  QCOMPARE(preprocess("#if 1lul\n #endif\n").trimmed(), QString("*ERROR*"));
+
+  QCOMPARE(preprocess("#if (2147483647L + 10L) > 0\n 10\n #endif\n").trimmed(), QString("10"));
+
   QCOMPARE(preprocess("#ifdef\n"), QString("*ERROR*"));
   
   QEXPECT_FAIL("", "Backslash incorrectly handled", Continue);
@@ -506,6 +530,9 @@ void TestParser::testPreprocessor() {
   
   QEXPECT_FAIL("", "Variadic macros unsupported", Continue);
   QCOMPARE(preprocess("#define PUT_BETWEEN(x,y) x y x\n#define NC(...) __VA_ARGS__\nPUT_BETWEEN(NC(pair<a,b>), c)\n").replace(QRegExp("[\n\t ]+"), " ").trimmed(), QString("pair<a,b> c pair<a,b>"));
+
+  QEXPECT_FAIL("", "Variadic arguments cannot be left empty (GCC extension)", Continue);
+  QCOMPARE(preprocess("#define NC(x,y...) x y\nNC(kde,ve)\nNC(lop)").replace(QRegExp("[\n\t ]+"), ""), QString("kdevelop"));
   
   QEXPECT_FAIL("", "No problems reported for missmatching macro-parameter-lists", Continue);
   QCOMPARE(preprocess("#define bla(x,y)\nbla(1,2,3)\n"), QString("*ERROR*"));
@@ -723,6 +750,44 @@ void TestParser::testQProperty()
   QCOMPARE(propAst->final, isFinal);
 }
 
+void TestParser::testDesignatedInitializers()
+{
+  TranslationUnitAST* ast;
+  InitializerListAST* listAst;
+  //DumpTree dumper;
+
+  ast = parse("\nA a = {"
+              "\n  .b = {"
+              "\n    .a = 10,"
+              "\n  },"
+              "\n  .x = 10,"
+              "\n  .y = SOME_CONST,"
+              "\n  .z = 10,"
+              "\n};");
+
+  QVERIFY(ast != 0);
+  QVERIFY(control.problems().isEmpty());
+  QCOMPARE(ast->declarations->count(), 1);
+  QVERIFY(hasKind(ast, AST::Kind_InitializerList));
+  listAst = static_cast<InitializerListAST*>(getAST(ast, AST::Kind_InitializerList));
+  QVERIFY(hasKind(listAst, AST::Kind_ClassMemberAccess));
+  //dumper.dump(ast, lastSession->token_stream);
+
+  ast = parse("\nint ia[10][5] = {"
+              "\n  [1] = 10,"
+              "\n  [2][B] = SOME_CONST,"
+              "\n};");
+
+  QVERIFY(ast != 0);
+  QVERIFY(control.problems().isEmpty());
+  QCOMPARE(ast->declarations->count(), 1);
+  QVERIFY(hasKind(ast, AST::Kind_InitializerList));
+  listAst = static_cast<InitializerListAST*>(getAST(ast, AST::Kind_InitializerList));
+  QVERIFY(hasKind(listAst, AST::Kind_SubscriptExpression));
+  //dumper.dump(ast, lastSession->token_stream);
+
+}
+
 void TestParser::testCommentAfterFunctionCall() {
   //this is ambigous
   TranslationUnitAST* ast = parse("void TestParser::setView() {\n"
@@ -880,11 +945,11 @@ void TestParser::testMultiByteCStrings()
   QVERIFY(control.problems().isEmpty());
   AST* str = getAST(ast, AST::Kind_StringLiteral);
   QVERIFY(str);
-  QCOMPARE(stringForNode(str), QString::fromUtf8("\"ä\""));
+  QCOMPARE(lastSession->stringForNode(str, true), QString::fromUtf8("\"ä\""));
   Token token = lastSession->token_stream->token(str->start_token);
   QEXPECT_FAIL("", "the wide ä-char takes two indizes in a QByteArray, which breaks our lexer", Abort);
   QCOMPARE(token.size, 3u);
-  QCOMPARE(token.symbolLength(), 3u);
+  QCOMPARE(lastSession->token_stream->symbolLength(token), 3u);
   Token endToken = lastSession->token_stream->token(str->end_token);
   rpp::Anchor pos = lastSession->positionAt(endToken.position);
   // should end just before the semicolon
@@ -984,15 +1049,6 @@ AST* TestParser::getAST(AST* ast, AST::NODE_KIND kind, int num)
   return visitor.ast;
 }
 
-QString TestParser::stringForNode(AST* node) const
-{
-  QString ret;
-  for(int i = node->start_token; i < node->end_token; ++i) {
-    ret += lastSession->token_stream->token(i).symbolString();
-  }
-  return ret;
-}
-
 #include "test_parser.moc"
 
 QTEST_MAIN(TestParser)
