@@ -37,9 +37,9 @@
 #include <language/backgroundparser/backgroundparser.h>
 #include <language/codegen/coderepresentation.h>
 #include <language/duchain/duchain.h>
+#include <language/duchain/duchainutils.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/topducontext.h>
-#include <language/editor/editorintegrator.h>
 #include <language/interfaces/iproblem.h>
 
 #include <threadweaver/Thread.h>
@@ -61,6 +61,7 @@
 
 // #define ifDebug(x) x
 #include "cpputils.h"
+#include <rpp/pp-location.h>
 
 const uint maxIncludeDepth = 50;
 
@@ -107,7 +108,7 @@ void PreprocessJob::foundHeaderGuard(rpp::Stream& stream, KDevelop::IndexedStrin
   m_currentEnvironment->environmentFile()->setHeaderGuard(guardName);
   
   //In naive matching mode, we ignore the dependence on header-guards
-  if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive)
+  if(Cpp::EnvironmentManager::self()->matchingLevel() <= Cpp::EnvironmentManager::Naive)
     m_currentEnvironment->removeString(guardName);
 }
 
@@ -127,6 +128,8 @@ void PreprocessJob::run()
     //kDebug(9007) << "Started pp job" << this << "parse" << parentJob()->parseJob() << "parent" << parentJob();
 
     kDebug(9007) << "PreprocessJob: preprocessing" << parentJob()->document().str();
+    if( parentJob()->parentPreprocessor() )
+      kDebug(9007) << "PARENT:" << parentJob()->parentPreprocessor()->parentJob()->document().toUrl();
 
     if (checkAbort())
         return;
@@ -134,11 +137,10 @@ void PreprocessJob::run()
     {
       KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
       
-      if(Cpp::EnvironmentManager::isSimplifiedMatching()) {
+      if(Cpp::EnvironmentManager::self()->isSimplifiedMatching()) {
         //Make sure that proxy-contexts and content-contexts never have the same identity, even if they have the same content.
         m_firstEnvironmentFile->setIdentityOffset(1); //Mark the first environment-file as the proxy
-        IndexedString u = parentJob()->document();
-        m_secondEnvironmentFile = new Cpp::EnvironmentFile(  u, 0 );
+        m_secondEnvironmentFile = new Cpp::EnvironmentFile( parentJob()->document(), 0 );
       }
     }
 
@@ -175,7 +177,9 @@ void PreprocessJob::run()
         
         if(parentJob()->masterJob() == parentJob() && updatingEnvironmentFile) {
           //Check whether we need to run at all, or whether the file is already up to date
-          if(updatingEnvironmentFile->featuresSatisfied(parentJob()->minimumFeatures()) && updatingEnvironmentFile->featuresSatisfied(parentJob()->slaveMinimumFeatures())) {
+          bool fileFeaturesSatisfied = updatingEnvironmentFile->featuresSatisfied(parentJob()->minimumFeatures());
+          bool slaveFeaturesSatisfied = updatingEnvironmentFile->featuresSatisfied(parentJob()->slaveMinimumFeatures());
+          if(fileFeaturesSatisfied && slaveFeaturesSatisfied) {
             KUrl localPath(parentJob()->document().toUrl());
             localPath.setFileName(QString());
             Cpp::EnvironmentFile* cppEnv = dynamic_cast<Cpp::EnvironmentFile*>(updatingEnvironmentFile.data());
@@ -197,6 +201,9 @@ void PreprocessJob::run()
               parentJob()->setNeedsUpdate(false);
               return;
             }
+          } else if (fileFeaturesSatisfied && !slaveFeaturesSatisfied) {
+            // This is strange, because importees should not require more features than the importer
+            kDebug() << "Slaves require more features than file itself" << parentJob()->slaveMinimumFeatures() << updatingEnvironmentFile->features() << parentJob()->document().toUrl();
           }
         }
     }
@@ -237,9 +244,12 @@ void PreprocessJob::run()
 
     PreprocessedContents result = preprocessor.processFile(parentJob()->document().str(), m_contents);
 
-    if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive && !m_headerSectionEnded && !m_firstEnvironmentFile->headerGuard().isEmpty()) {
+    if(Cpp::EnvironmentManager::self()->matchingLevel() <= Cpp::EnvironmentManager::Naive && !m_headerSectionEnded && !m_firstEnvironmentFile->headerGuard().isEmpty()) {
       if(macroNamesAtBeginning.contains(m_firstEnvironmentFile->headerGuard())) {
         //Remove the header-guard, and re-preprocess, since we don't do real environment-management(We don't allow empty versions)
+        // We also have to clear the location-table here, because it already contains 'wrong' contents from the previous preprocessing
+        kDebug() << "Re-processing header with the header-guard disabled";
+        delete m_currentEnvironment->takeLocationTable();
         m_currentEnvironment->removeMacro(m_firstEnvironmentFile->headerGuard());
         result = preprocessor.processFile(parentJob()->document().str(), m_contents);
       }
@@ -306,23 +316,6 @@ void PreprocessJob::headerSectionEnded(rpp::Stream& stream)
   headerSectionEndedInternal(&stream);
 }
 
-TopDUContext* contentFromProxy(TopDUContext* ctx) {
-    if( ctx->parsingEnvironmentFile() && ctx->parsingEnvironmentFile()->isProxyContext() ) {
-        {
-          ReferencedTopDUContext ref(ctx);
-        }
-        if(ctx->importedParentContexts().isEmpty()) {
-          kDebug() << "proxy-context for" << ctx->url().str() << "has no imports!" << ctx->ownIndex();
-          return 0;
-        }
-        Q_ASSERT(!ctx->importedParentContexts().isEmpty());
-        return dynamic_cast<TopDUContext*>(ctx->importedParentContexts().first().context(0));
-    }else{
-        return ctx;
-    }
-}
-
-
 void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
 {
     bool closeStream = false;
@@ -352,7 +345,7 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
         KDevelop::ReferencedTopDUContext content;
 
         if(m_updatingEnvironmentFile)
-          content = KDevelop::ReferencedTopDUContext(contentFromProxy(m_updatingEnvironmentFile->topContext()));
+          content = KDevelop::ReferencedTopDUContext(DUChainUtils::contentContextFromProxyContext(m_updatingEnvironmentFile->topContext()));
         else
           content = KDevelop::DUChain::self()->chainForDocument(u, m_currentEnvironment, false);
 
@@ -378,7 +371,7 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
             localPath.setFileName(QString());
             
             if(contentEnvironment->matchEnvironment(m_currentEnvironment) && !CppUtils::needsUpdate(contentEnvironment, localPath, parentJob()->includePathUrls()) && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(content)) && (content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->minimumFeatures()) && content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->slaveMinimumFeatures())) 
-              && Cpp::EnvironmentManager::matchingLevel() != Cpp::EnvironmentManager::Disabled) {
+              && Cpp::EnvironmentManager::self()->matchingLevel() != Cpp::EnvironmentManager::Disabled) {
               ///@todo We never keep the duchain while updating now in disabled environment matching mode.
               ///           We don't need it there, and changes in imports may be simply ignored when the keeping is enabled.
               ///           However when full environment management is enabled this is needed, as the same content may be shared for multiple proxy contexts.
@@ -449,6 +442,8 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
     if (skipCurrentPath)
       from = parentJob()->includedFromPath();
 
+
+    ///FIXME: use IndexedStrings
     QPair<KUrl, KUrl> included = CppUtils::findInclude( parentJob()->includePathUrls(), localPath, fileName, type, from );
     KUrl includedFile = included.first;
     if (includedFile.isValid()) {
@@ -463,7 +458,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
               KDevelop::ProblemPointer p(new Problem()); ///@todo create special include-problem
               p->setSource(KDevelop::ProblemData::Preprocessor);
               p->setDescription(i18n("File was included recursively from within itself: %1", fileName ));
-              p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor(sourceLine,0), KTextEditor::Cursor(sourceLine+1,0)));
+              p->setFinalLocation(DocumentRange(parentJob()->document(), SimpleRange(sourceLine,0, sourceLine+1,0)));
               p->setLocationStack(parentJob()->includeStack());
               parentJob()->addPreprocessorProblem(p);
               return 0;
@@ -482,8 +477,8 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
             KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
             includedContext = KDevelop::DUChain::self()->chainForDocument(includedFile, m_currentEnvironment, (bool)m_secondEnvironmentFile);
             
-            //Check if the same file is being processed by one of the parents, and if it is, import it later on
-            if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive) {
+            //Check if the same file _is_ one of the parents, and if it is, import it later on
+            if(Cpp::EnvironmentManager::self()->matchingLevel() <= Cpp::EnvironmentManager::Naive) {
               
               CPPParseJob* job = parentJob();
               while(job->parentPreprocessor()) {
@@ -522,7 +517,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
                 
                 #if 0
                 //If header-guards should be ignored, unguard the file
-                if(Cpp::EnvironmentManager::ignoreGuardsForImporting() &&
+                if(Cpp::EnvironmentManager::self()->ignoreGuardsForImporting() &&
                   !includedEnvironment->headerGuard().isEmpty() && m_currentEnvironment->macroNameSet().contains(includedEnvironment->headerGuard()))
                 {
                   m_currentEnvironment->removeMacro(includedEnvironment->headerGuard());
@@ -574,13 +569,13 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
 
             ///The second parameter is zero because we are in a background-thread and we here
             ///cannot create a slave of the foreground cpp-support-part.
-            CPPParseJob* slaveJob = new CPPParseJob(includedFile, this);
+            CPPParseJob* slaveJob = new CPPParseJob(IndexedString(includedFile), parentJob()->cpp(), this);
             
             slaveJob->setMinimumFeatures(slaveMinimumFeatures);
 
             slaveJob->setIncludedFromPath(included.second);
 
-            includeStack.append(DocumentCursor(HashedString(parentJob()->document().str()), KTextEditor::Cursor(sourceLine, 0)));
+            includeStack.append(DocumentCursor(parentJob()->document(), SimpleCursor(sourceLine, 0)));
             slaveJob->setIncludeStack(includeStack);
 
             slaveJob->parseForeground();
@@ -610,7 +605,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
         p->setSource(KDevelop::ProblemData::Preprocessor);
         p->setDescription(i18n("Included file was not found: %1", fileName ));
         p->setExplanation(i18n("Searched include path:\n%1", urlsToString(parentJob()->includePathUrls())));
-        p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor(sourceLine,0), KTextEditor::Cursor(sourceLine+1,0)));
+        p->setFinalLocation(DocumentRange(parentJob()->document(), SimpleRange(sourceLine,0, sourceLine+1,0)));
         p->setLocationStack(parentJob()->includeStack());
         p->setSolutionAssistant(KSharedPtr<KDevelop::IAssistant>(new Cpp::MissingIncludePathAssistant(parentJob()->masterJob()->document(), _fileName)));
         parentJob()->addPreprocessorProblem(p);
@@ -655,71 +650,21 @@ bool PreprocessJob::checkAbort()
 
 bool PreprocessJob::readContents()
 {
-    bool readFromDisk = !parentJob()->contentsAvailableFromEditor();
-    parentJob()->setReadFromDisk(readFromDisk);
-    
-    QString localFile(parentJob()->document().toUrl().toLocalFile());
+  KDevelop::ProblemPointer p = parentJob()->readContents();
   
-    QFileInfo fileInfo( localFile );
-    
-    if ( readFromDisk )
-    {
-        QFile file( localFile );
-        if ( !file.open( QIODevice::ReadOnly ) )
-        {
-            //Try using a code-representation, as artificial code may have been inserted
-            if(artificialCodeRepresentationExists(parentJob()->document())) {
-              CodeRepresentation::Ptr repr = createCodeRepresentation(parentJob()->document());
-              m_contents = repr->text().toUtf8();
-              kDebug() << "took contents for " << parentJob()->document().toUrl() << " from code-representation:\n" << m_contents;
-              return true;
-            }
-          
-            KDevelop::ProblemPointer p(new Problem());
-            p->setSource(KDevelop::ProblemData::Disk);
-            p->setDescription(i18n( "Could not open file '%1'", localFile ));
-            switch (file.error()) {
-              case QFile::ReadError:
-                  p->setExplanation(i18n("File could not be read from."));
-                  break;
-              case QFile::OpenError:
-                  p->setExplanation(i18n("File could not be opened."));
-                  break;
-              case QFile::PermissionsError:
-                  p->setExplanation(i18n("File permissions prevent opening for read."));
-                  break;
-              default:
-                  break;
-            }
-            p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor::invalid(), KTextEditor::Cursor::invalid()));
-            p->setLocationStack(parentJob()->includeStack());
-            parentJob()->addPreprocessorProblem(p);
+  if(p)
+  {
+    p->setLocationStack(parentJob()->includeStack());
+    parentJob()->addPreprocessorProblem(p);
+    return false;
+  }
+  
+  m_firstEnvironmentFile->setModificationRevision( parentJob()->contents().modification );
 
-            kWarning( 9007 ) << "Could not open file" << parentJob()->document().str() << "(path" << localFile << ")" ;
-            return false;
-        }
-        m_contents = file.readAll(); ///@todo respect local encoding settings. Currently, the file is expected to be utf-8
-        file.close();
-        m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision(fileInfo.lastModified()) );
-    }
-    else{
-        ///This has to be done, before parentJob()->revisionToken() is read, as it sets the token.
-        m_contents = parentJob()->contentsFromEditor().toUtf8();
-        m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision( fileInfo.lastModified(), parentJob()->revisionToken() ) );
-    }
-    
-    ///@todo Modify parsing foronly changed ranges on editor files
-#if 0
-    //===--- Incremental Parsing!!! yay :) ---===//
-    kDebug() << "We could have just parsed the changed ranges:";
-    foreach (KTextEditor::SmartRange* range, parentJob()->changedRanges())
-        kDebug() << *range << range->text().join("\n").left(20) << "...";
-#endif
+  m_contents = parentJob()->contents().contents;
     
     ifDebug( kDebug( 9007 ) << "===-- PREPROCESSING --===> "
     << parentJob()->document().str()
-    << "<== readFromDisk:" << readFromDisk
-    << "size:" << contents.length()
     << endl; )
     
     return true;
