@@ -35,6 +35,20 @@
 #define CMAKE_REGISTER_AST( klassName, astId ) namespace {                 \
         CMakeAst* Create##klassName() { return new klassName; }            \
         bool b_##astId = AstFactory::self()->registerAst( QLatin1String( #astId ), Create##klassName ); }
+        
+QDebug operator<<(QDebug dbg, const CMakeFunctionDesc &func)
+{
+    dbg.nospace() << func.name << "(" << func.arguments << ")";
+
+    return dbg.space();
+}
+
+QDebug operator<<(QDebug dbg, const CMakeFunctionArgument &arg)
+{
+    dbg.nospace() << arg.value;
+
+    return dbg.space();
+}
 
 CMAKE_REGISTER_AST( AddDefinitionsAst, add_definitions )
 CMAKE_REGISTER_AST( AddDependenciesAst, add_dependencies )
@@ -113,6 +127,7 @@ CMAKE_REGISTER_AST( TryCompileAst, try_compile )
 CMAKE_REGISTER_AST( TryRunAst, try_run )
 CMAKE_REGISTER_AST( UseMangledMesaAst, use_mangled_mesa )
 CMAKE_REGISTER_AST( UtilitySourceAst, utility_source )
+CMAKE_REGISTER_AST( UnsetAst, unset )
 CMAKE_REGISTER_AST( VariableRequiresAst, variable_requires )
 CMAKE_REGISTER_AST( WhileAst, while)
 CMAKE_REGISTER_AST( WriteFileAst, write_file)
@@ -277,7 +292,7 @@ bool CustomTargetAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 
     //check and make sure the target name isn't something silly
     CMakeFunctionArgument arg = func.arguments.front();
-    if ( arg.value.toLower() == QLatin1String( "all" ) )
+    if ( arg.value=="ALL" )
         return false;
     else
         m_target = arg.value;
@@ -286,10 +301,7 @@ bool CustomTargetAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     if(func.arguments.count()>1)
     {
         CMakeFunctionArgument arg2 = func.arguments[1];
-        if ( arg2.value.toUpper() == QLatin1String( "ALL" ) )
-            m_buildAlways = true;
-        else
-            m_buildAlways = false;
+        m_buildAlways = arg2.value == "ALL";
     }
 
     //what are we doing?
@@ -378,7 +390,7 @@ AddDefinitionsAst::~AddDefinitionsAst()
 
 bool AddDefinitionsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "add_definitions" )
+    if ( func.name != "add_definitions" )
         return false;
 
     if ( func.arguments.isEmpty() )
@@ -404,7 +416,7 @@ AddDependenciesAst::~AddDependenciesAst()
 
 bool AddDependenciesAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "add_dependencies" )
+    if ( func.name != "add_dependencies" )
         return false;
 
     if ( func.arguments.size() < 2 )
@@ -437,7 +449,7 @@ AddExecutableAst::~AddExecutableAst()
 
 bool AddExecutableAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "add_executable" )
+    if ( func.name != "add_executable" )
         return false;
 
     if ( func.arguments.size() < 2 )
@@ -490,7 +502,7 @@ AddLibraryAst::~AddLibraryAst()
 
 bool AddLibraryAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "add_library" || func.arguments.isEmpty() )
+    if ( func.name != "add_library" || func.arguments.isEmpty() )
         return false;
 
     bool libTypeSet = false;
@@ -545,7 +557,7 @@ AddSubdirectoryAst::~AddSubdirectoryAst()
 
 bool AddSubdirectoryAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "add_subdirectory" )
+    if ( func.name != "add_subdirectory" )
         return false;
     if ( func.arguments.size() < 1 || func.arguments[0].value.isEmpty())
         return false;
@@ -576,19 +588,42 @@ AddTestAst::~AddTestAst()
 
 bool AddTestAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "add_test" )
+    if ( func.name != "add_test" )
         return false;
     if ( func.arguments.size() < 2 )
         return false;
 
-    m_testName = func.arguments[0].value;
-    m_exeName = func.arguments[1].value;
     QList<CMakeFunctionArgument>::const_iterator it, itEnd = func.arguments.constEnd();
-    it = func.arguments.constBegin() + 2;
+    it = func.arguments.constBegin();
+    enum {Name, Command, Arg, Unsupported} state = Unsupported;
+    if (it->value != "NAME")
+    {
+        m_testName = (it++)->value;
+        m_exeName = (it++)->value;
+        state = Arg;
+    }
     for ( ; it != itEnd; ++it )
-        m_testArgs << it->value;
+    {
+        if ( it->value == "NAME" ) state = Name;
+        else if ( it->value == "COMMAND" ) state = Command;
+        else if ( it->value == "CONFIGURATIONS" || it->value == "WORKING_DIRECTORY" ) state = Unsupported;
+        else switch (state) {
+            case Name:
+                m_testName = it->value;
+                break;
+            case Command:
+                m_exeName = it->value;
+                state = Arg;
+                break;
+            case Arg:
+                m_testArgs << it->value;
+                break;
+            default:
+                break;
+        }
+    }
 
-    return true;
+    return !m_exeName.isEmpty();
 }
 
 AuxSourceDirectoryAst::AuxSourceDirectoryAst()
@@ -601,7 +636,7 @@ AuxSourceDirectoryAst::~AuxSourceDirectoryAst()
 
 bool AuxSourceDirectoryAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "aux_source_directory" )
+    if ( func.name != "aux_source_directory" )
         return false;
     if ( func.arguments.size() != 2 )
         return false;
@@ -623,7 +658,7 @@ BuildCommandAst::~BuildCommandAst()
 
 bool BuildCommandAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "build_command" )
+    if ( func.name != "build_command" )
         return false;
     if ( func.arguments.size() < 2 )
         return false;
@@ -645,7 +680,7 @@ BuildNameAst::~BuildNameAst()
 
 bool BuildNameAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "build_name" )
+    if ( func.name != "build_name" )
         return false;
     if ( func.arguments.size() != 1 )
         return false;
@@ -667,9 +702,9 @@ CMakeMinimumRequiredAst::~CMakeMinimumRequiredAst()
 
 bool CMakeMinimumRequiredAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "cmake_minimum_required" )
+    if ( func.name != "cmake_minimum_required" )
         return false;
-    if ( func.arguments.size() < 2 || func.arguments.count() > 3 || func.arguments.first().value.toUpper() != "VERSION")
+    if ( func.arguments.size() < 2 || func.arguments.count() > 3 || func.arguments.first().value != "VERSION")
         return false;
 
     bool correct = false;
@@ -679,7 +714,7 @@ bool CMakeMinimumRequiredAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 
     if(func.arguments.count()==3)
     {
-        if(func.arguments[2].value.toUpper()=="FATAL_ERROR")
+        if(func.arguments[2].value=="FATAL_ERROR")
             m_wrongVersionIsFatal = true;
         else
             return false;
@@ -698,7 +733,7 @@ ConfigureFileAst::~ConfigureFileAst()
 
 bool ConfigureFileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "configure_file" )
+    if ( func.name != "configure_file" )
         return false;
 
     if ( func.arguments.size() < 2 )
@@ -737,7 +772,7 @@ CreateTestSourcelistAst::~CreateTestSourcelistAst()
 
 bool CreateTestSourcelistAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "create_test_sourcelist" )
+    if ( func.name != "create_test_sourcelist" )
         return false;
 
     if ( func.arguments.count() < 3 )
@@ -784,7 +819,7 @@ EnableLanguageAst::~EnableLanguageAst()
 
 bool EnableLanguageAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "enable_language" )
+    if ( func.name != "enable_language" )
         return false;
 
     if ( func.arguments.isEmpty() || func.arguments.count() != 1 )
@@ -822,7 +857,7 @@ ExecProgramAst::~ExecProgramAst()
 
 bool ExecProgramAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if(func.name.toLower() != "exec_program" || func.arguments.count() == 0 )
+    if(func.name != "exec_program" || func.arguments.count() == 0 )
         return false;
 
     m_executableName = func.arguments[0].value;
@@ -832,7 +867,7 @@ bool ExecProgramAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     // don't re-read the first element
     it++;
     for(; it!=itEnd; ++it) {
-        if(it->value.toUpper()=="OUTPUT_VARIABLE") {
+        if(it->value=="OUTPUT_VARIABLE") {
             ++it;
             if(it!=itEnd)
             {
@@ -841,7 +876,7 @@ bool ExecProgramAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             }
             else
                 return false;
-        } else if(it->value.toUpper()=="RETURN_VALUE") {
+        } else if(it->value=="RETURN_VALUE") {
             ++it;
             if(it!=itEnd)
             {
@@ -851,7 +886,7 @@ bool ExecProgramAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             else
                 return false;
         }
-        else if(it->value.toUpper()=="ARGS")
+        else if(it->value=="ARGS")
         {
             args=true;
         }
@@ -878,7 +913,7 @@ ExecuteProcessAst::~ExecuteProcessAst()
 
 bool ExecuteProcessAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if(func.name.toLower() != "execute_process" || func.arguments.count()<2)
+    if(func.name != "execute_process" || func.arguments.count()<2)
         return false;
     
     enum Action {
@@ -990,7 +1025,7 @@ ExportLibraryDepsAst::~ExportLibraryDepsAst()
 
 bool ExportLibraryDepsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if(func.name.toLower() != "export_library_dependencies" || func.arguments.isEmpty())
+    if(func.name != "export_library_dependencies" || func.arguments.isEmpty())
         return false;
     m_file=func.arguments[0].value;
     if(func.arguments.count()>=2)
@@ -1006,7 +1041,7 @@ bool ExportLibraryDepsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 }
 
 FileAst::FileAst()
-    : m_newlineConsume(false), m_noHexConversion(false)
+    : m_isFollowingSymlinks(false), m_newlineConsume(false), m_noHexConversion(false)
 {
 }
 
@@ -1017,7 +1052,7 @@ FileAst::~FileAst()
 
 bool FileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "file" || func.arguments.count()<2)
+    if ( func.name != "file" || func.arguments.count()<2)
         return false;
 
     QString type = func.arguments.first().value;
@@ -1267,6 +1302,8 @@ bool FindFileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             s=PATHS;
         else if(it->value=="PATH_SUFFIXES")
             s=PATH_SUFFIXES;
+        else if(it->value=="HINTS")
+            s=HINTS;
         else switch(s) {
             case NAMES:
                 m_filenames << it->value;
@@ -1284,7 +1321,7 @@ bool FindFileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
                 break;
         }
     }
-    return !m_filenames.isEmpty() && !m_path.isEmpty();
+    return !m_filenames.isEmpty();
 }
 
 
@@ -1311,7 +1348,7 @@ bool MacroCallAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 }
 
 FindLibraryAst::FindLibraryAst()
-    : m_noDefaultPath(false)
+    : m_noDefaultPath(false), m_cmakeFindRootPath(false)
 {
 }
 
@@ -1347,6 +1384,8 @@ bool FindLibraryAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             m_noCmakeEnvironmentPath = true;
         else if(it->value=="NO_CMAKE_PATH")
             m_noSystemEnvironmentPath = true;
+        else if(it->value=="NO_CMAKE_FIND_ROOT_PATH")
+            m_cmakeFindRootPath = true;
         else if(it->value=="DOC") {
             ++it;
             if(it==itEnd)
@@ -1392,7 +1431,7 @@ bool FindPackageAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     m_isQuiet=false;
     m_noModule=false;
     m_isRequired=false;
-    if ( func.name.toLower() != "find_package" )
+    if ( func.name != "find_package" )
         return false;
 
     if ( func.arguments.isEmpty() )
@@ -1402,22 +1441,27 @@ bool FindPackageAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     QList<CMakeFunctionArgument>::const_iterator it=func.arguments.constBegin()+1;
     QList<CMakeFunctionArgument>::const_iterator itEnd=func.arguments.constEnd();
 
+    bool ret=true;
+    enum State { None, Components, Paths };
+    State s=None;
     for(; it!=itEnd; ++it)
     {
         if(it->value.isEmpty())
         {}
-        else if(it->value[0].isNumber())
-        {
-            m_version=it->value;
-        }
+        else if(it->value[0].isNumber()) m_version=it->value;
+        else if(it->value=="QUIET") m_isQuiet=true;
+        else if(it->value=="NO_MODULE") m_noModule=true;
+        else if(it->value=="REQUIRED") { m_isRequired=true; s=Components; }
+        else if(it->value=="COMPONENTS") s=Components;
+        else if(it->value=="PATHS") s=Paths;
+        else if(s==Components)
+            m_components.append(it->value);
+        else if(s==Paths)
+            m_paths.append(it->value);
         else
-        {
-            m_isQuiet |= it->value=="QUIET";
-            m_noModule |= it->value=="NO_MODULE";
-            m_isRequired |= it->value=="REQUIRED";
-        }
+            ret=false;
     }
-    return true;
+    return ret;
 }
 
 FindPathAst::FindPathAst()
@@ -1461,6 +1505,10 @@ bool FindPathAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         else if(it->value=="NO_CMAKE_ENVIRONMENT_PATH")
             m_noCmakeEnvironmentPath = true;
         else if(it->value=="NO_CMAKE_PATH")
+            m_noCmakePath = true;
+        else if(it->value=="NO_CMAKE_SYSTEM_PATH")
+            m_noCmakeSystemPath = true;
+        else if(it->value=="NO_SYSTEM_ENVIRONMENT_PATH")
             m_noSystemEnvironmentPath = true;
         else if(it->value=="DOC") {
             ++it;
@@ -1496,7 +1544,7 @@ bool FindPathAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 FindProgramAst::FindProgramAst()
 {
         m_noDefaultPath = m_noCmakeEnvironmentPath = m_noCmakePath =
-        m_noSystemEnvironmentPath = false;
+        m_noSystemEnvironmentPath = m_noCMakeFindRootPath = false;
 }
 
 FindProgramAst::~FindProgramAst()
@@ -1530,8 +1578,8 @@ bool FindProgramAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             m_noCmakePath=true;
         else if(it->value=="NO_SYSTEM_ENVIRONMENT_PATH")
             m_noSystemEnvironmentPath = true;
-        else if(it->value=="NO_SYSTEM_ENVIRONMENT_PATH")
-            m_noSystemEnvironmentPath = true;
+        else if(it->value=="NO_CMAKE_FIND_ROOT_PATH")
+            m_noCMakeFindRootPath = true;
         else if(it->value=="DOC") {
             ++it;
             if(it==itEnd)
@@ -1587,13 +1635,13 @@ ForeachAst::~ForeachAst()
 
 bool ForeachAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if(func.name.toLower()!="foreach" || func.arguments.count()<2)
+    if(func.name.toLower()!="foreach" || func.arguments.count()<1)
         return false;
     addOutputArgument(func.arguments.first());
     m_loopVar=func.arguments.first().value;
-    if(func.arguments[1].value=="RANGE") {
+    if(func.arguments.count()>1 && func.arguments[1].value=="RANGE") {
         bool correctStart = true, correctStop = true, correctRange = true;
-        m_range=true;
+        m_type=Range;
         if(func.arguments.count()<3)
             return false;
         m_ranges.step = 1;
@@ -1611,10 +1659,21 @@ bool ForeachAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         if(!correctStart || !correctStop || !correctRange)
             return false;
     } else {
-        m_range=false;
-        QList<CMakeFunctionArgument>::const_iterator it=func.arguments.constBegin()+1, itEnd=func.arguments.constEnd();
-        for(; it!=itEnd; ++it)
-        {
+        int incr;
+        if(func.arguments.count()>1 && func.arguments[1].value=="IN") {
+            if(func.arguments[2].value=="LISTS") {
+                m_type = InLists;
+            } else if(func.arguments[2].value=="ITEMS") {
+                m_type = InItems;
+            } else
+                return false;
+            incr=3;
+        } else {
+            m_type=InItems;
+            incr=1;
+        }
+        QList<CMakeFunctionArgument>::const_iterator it=func.arguments.constBegin()+incr, itEnd=func.arguments.constEnd();
+        for(; it!=itEnd; ++it) {
             m_arguments += it->value;
         }
     }
@@ -1637,7 +1696,7 @@ bool GetCMakePropertyAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     addOutputArgument(func.arguments[0]);
     m_variableName = func.arguments[0].value;
     
-    QString type=func.arguments[1].value.toUpper();
+    QString type=func.arguments[1].value;
     if(type=="VARIABLES")
         m_type=Variables;
     else if(type=="CACHE_VARIABLES")
@@ -1646,6 +1705,8 @@ bool GetCMakePropertyAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         m_type=Commands;
     else if(type=="MACROS")
         m_type=Macros;
+    else if(type=="COMPONENTS")
+        m_type=Components;
     else
         return false;
     return true;
@@ -1809,7 +1870,7 @@ IncludeAst::~IncludeAst()
 
 bool IncludeAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "include" || (func.arguments.isEmpty() || func.arguments.size() > 4))
+    if ( func.name != "include" || (func.arguments.isEmpty() || func.arguments.size() > 4))
         return false;
 
     m_includeFile = func.arguments[0].value;
@@ -1846,28 +1907,22 @@ IncludeDirectoriesAst::~IncludeDirectoriesAst()
 
 bool IncludeDirectoriesAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "include_directories" || func.arguments.isEmpty() )
+    if ( func.name != "include_directories" || func.arguments.isEmpty() )
         return false;
     
     int i=0;
     m_includeType = Default;
     m_isSystem = false;
     
-    if(func.arguments[i].value=="AFTER") {
-        if(m_includeType!=Default)
-            return false;
+    if(func.arguments.first().value=="AFTER") {
         m_includeType = After;
         i++;
-    }
-    
-    if(func.arguments[i].value=="BEFORE") {
-        if(m_includeType!=Default)
-            return false;
+    } else if(func.arguments.first().value=="BEFORE") {
         m_includeType = Before;
         i++;
     }
 
-    if(func.arguments[i].value=="SYSTEM") {
+    if(i<func.arguments.size() && func.arguments[i].value=="SYSTEM") {
         m_isSystem = true;
         i++;
     }
@@ -1875,9 +1930,8 @@ bool IncludeDirectoriesAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     if ( func.arguments.size() <= i )
         return false;
 
-    QList<CMakeFunctionArgument>::const_iterator it, itEnd;
-    it=func.arguments.constBegin() + i;
-    itEnd = func.arguments.constEnd();
+    QList<CMakeFunctionArgument>::const_iterator it=func.arguments.constBegin() + i;
+    QList<CMakeFunctionArgument>::const_iterator itEnd = func.arguments.constEnd();
 
     for ( ; it != itEnd; ++it )
         m_includedDirectories.append(it->value);
@@ -1911,7 +1965,7 @@ IncludeRegularExpressionAst::~IncludeRegularExpressionAst()
 
 bool IncludeRegularExpressionAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "include_regular_expression" || func.arguments.isEmpty() || func.arguments.count()>2 )
+    if ( func.name != "include_regular_expression" || func.arguments.isEmpty() || func.arguments.count()>2 )
         return false;
     m_match=func.arguments[0].value;
     if(func.arguments.count()==2)
@@ -2038,7 +2092,7 @@ LinkDirectoriesAst::~LinkDirectoriesAst()
 
 bool LinkDirectoriesAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "include_regular_expression" || func.arguments.isEmpty() )
+    if ( func.name != "link_directories" || func.arguments.isEmpty() )
         return false;
     
     foreach(const CMakeFunctionArgument &arg, func.arguments)
@@ -2057,7 +2111,7 @@ LinkLibrariesAst::~LinkLibrariesAst()
 
 bool LinkLibrariesAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "link_libraries" || func.arguments.isEmpty() )
+    if ( func.name != "link_libraries" || func.arguments.isEmpty() )
         return false;
     
     QString lastLib;
@@ -2091,7 +2145,7 @@ ListAst::~ListAst()
 
 bool ListAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "list" )
+    if ( func.name != "list" )
         return false;
     if ( func.arguments.size() < 2)
         return false;
@@ -2156,6 +2210,7 @@ bool ListAst::parseFunctionInfo( const CMakeFunctionDesc& func )
                 return false;
             
             addOutputArgument(func.arguments[1]);
+            m_output = func.arguments[1].value;
             int i=0;
             foreach(const CMakeFunctionArgument& arg, func.arguments)
             {
@@ -2192,6 +2247,7 @@ bool ListAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             if(func.arguments.count()<3)
                 return false;
             addOutputArgument(func.arguments[1]);
+            m_output = func.arguments[1].value;
             
             QList<CMakeFunctionArgument>::const_iterator it=func.arguments.constBegin()+2, itEnd=func.arguments.constEnd();
             for(; it!=itEnd; ++it)
@@ -2203,6 +2259,7 @@ bool ListAst::parseFunctionInfo( const CMakeFunctionDesc& func )
             if(func.arguments.count()<3)
                 return false;
             addOutputArgument(func.arguments[1]);
+            m_output = func.arguments[1].value;
             int i=0;
             foreach(const CMakeFunctionArgument& arg, func.arguments)
             {
@@ -2220,6 +2277,7 @@ bool ListAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         case Reverse:
         case RemoveDuplicates:
             addOutputArgument(func.arguments[1]);
+            m_output = func.arguments[1].value;
             if(func.arguments.count()>2)
                 return false;
             break;
@@ -2238,7 +2296,7 @@ LoadCacheAst::~LoadCacheAst()
 
 bool LoadCacheAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "load_cache" || func.arguments.count()<4)
+    if ( func.name != "load_cache" || func.arguments.count()<4)
         return false;
     m_cachePath=func.arguments[0].value;
     if(func.arguments[1].value=="READ_WITH_PREFIX")
@@ -2293,7 +2351,7 @@ LoadCommandAst::~LoadCommandAst()
 
 bool LoadCommandAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "load_command" || func.arguments.count()<4)
+    if ( func.name != "load_command" || func.arguments.count()<4)
         return false;
     m_cmdName=func.arguments[0].value;
     
@@ -2314,7 +2372,7 @@ MacroAst::~MacroAst()
 
 bool MacroAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "macro" || func.arguments.isEmpty())
+    if ( func.name != "macro" || func.arguments.isEmpty())
         return false;
 
     m_macroName = func.arguments.first().value.toLower();
@@ -2336,7 +2394,7 @@ FunctionAst::~FunctionAst()
 
 bool FunctionAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "function" || func.arguments.isEmpty())
+    if ( func.name != "function" || func.arguments.isEmpty())
         return false;
 
     m_name = func.arguments.first().value.toLower();
@@ -2358,7 +2416,7 @@ MakeDirectoryAst::~MakeDirectoryAst()
 
 bool MakeDirectoryAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "make_directory" || func.arguments.isEmpty() || func.arguments.size()>1)
+    if ( func.name != "make_directory" || func.arguments.isEmpty() || func.arguments.size()>1)
         return false;
     m_directory=func.arguments.first().value;
     return true;
@@ -2376,7 +2434,7 @@ MarkAsAdvancedAst::~MarkAsAdvancedAst()
 
 bool MarkAsAdvancedAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "mark_as_advanced" || func.arguments.isEmpty() )
+    if ( func.name != "mark_as_advanced" || func.arguments.isEmpty() )
         return false;
 
     m_isClear = func.arguments[0].value == "CLEAR";
@@ -2458,7 +2516,7 @@ OptionAst::~OptionAst()
 
 bool OptionAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if(func.name.toLower()!="option" || (func.arguments.count()<2 && func.arguments.count()>3))
+    if(func.name.toLower()!="option" || func.arguments.count()<2 || func.arguments.count()>3)
         return false;
     m_variableName = func.arguments[0].value;
     m_description = func.arguments[1].value;
@@ -2497,7 +2555,7 @@ ProjectAst::~ProjectAst()
 
 bool ProjectAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "project" )
+    if ( func.name != "project" )
         return false;
 
     if ( func.arguments.size() < 1 )
@@ -2563,7 +2621,7 @@ RemoveAst::~RemoveAst()
 
 bool RemoveAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "remove" || func.arguments.isEmpty())
+    if ( func.name != "remove" || func.arguments.isEmpty())
         return false;
 
     m_variableName = func.arguments[0].value;
@@ -2587,7 +2645,7 @@ RemoveDefinitionsAst::~RemoveDefinitionsAst()
 
 bool RemoveDefinitionsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "remove_definitions")
+    if ( func.name != "remove_definitions")
         return false;
 
     QList<CMakeFunctionArgument>::const_iterator it, itEnd = func.arguments.end();
@@ -2610,7 +2668,7 @@ SeparateArgumentsAst::~SeparateArgumentsAst()
 /// introduced in CMake 2.8.
 bool SeparateArgumentsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "separate_arguments" || func.arguments.count()!=1)
+    if ( func.name != "separate_arguments" || func.arguments.count()!=1)
         return false;
 
     addOutputArgument(func.arguments.first());
@@ -2630,7 +2688,7 @@ SetAst::~SetAst()
 
 bool SetAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "set" || func.arguments.isEmpty() )
+    if ( func.name != "set" || func.arguments.isEmpty() )
         return false;
 
     m_variableName = func.arguments.first().value;
@@ -2905,13 +2963,13 @@ bool StringAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
     if(func.name.toLower()!="string" || func.arguments.count()<2)
         return false;
-    QString stringType=func.arguments[0].value.toUpper();
+    QString stringType=func.arguments[0].value;
     if(stringType=="REGEX")
     {
         if(func.arguments.count()<5)
             return false;
         m_type=Regex;
-        QString regexType = func.arguments[1].value.toUpper();
+        QString regexType = func.arguments[1].value;
 
         int outpos=3;
         if(regexType=="MATCH") m_cmdType=Match;
@@ -2936,14 +2994,16 @@ bool StringAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     }
     else if(stringType=="REPLACE")
     {
+        if(func.arguments.count()<4)
+            return false;
+        
         m_type=Replace;
         m_regex = func.arguments[1].value;
         m_replace=func.arguments[2].value;
         m_outputVariable = func.arguments[3].value;
         addOutputArgument(func.arguments[3]);
         
-        QList<CMakeFunctionArgument>::const_iterator it=func.arguments.begin()+4;
-        QList<CMakeFunctionArgument>::const_iterator itEnd=func.arguments.end();
+        QList<CMakeFunctionArgument>::const_iterator it=func.arguments.begin()+4, itEnd=func.arguments.end();
         for(; it!=itEnd; ++it)
         {
             m_input += it->value;
@@ -2954,7 +3014,7 @@ bool StringAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         if(func.arguments.count()!=5)
             return false;
         m_type=Compare;
-        QString argumentType=func.arguments[1].value.toUpper();
+        QString argumentType=func.arguments[1].value;
         if(argumentType=="EQUAL") m_cmdType=Equal;
         else if(argumentType=="NOTEQUAL") m_cmdType=NotEqual;
         else if(argumentType=="LESS") m_cmdType=Less;
@@ -2988,29 +3048,29 @@ bool StringAst::parseFunctionInfo( const CMakeFunctionDesc& func )
         addOutputArgument(func.arguments[2]);
         
         int i=3;
-        if(func.arguments.count()>i) m_only = func.arguments[i].value.toUpper()=="@ONLY";
+        if(func.arguments.count()>i) m_only = func.arguments[i].value=="@ONLY";
         if(m_only) i++;
-        if(func.arguments.count()>i) m_escapeQuotes = func.arguments[i].value.toUpper()=="ESCAPE_QUOTES";
+        if(func.arguments.count()>i) m_escapeQuotes = func.arguments[i].value=="ESCAPE_QUOTES";
     }
-    else if(stringType=="TOUPPER")
+    else if(stringType=="TOUPPER" && func.arguments.count()==3)
     {
         m_type=ToUpper;
         m_input.append(func.arguments[1].value);
         m_outputVariable = func.arguments[2].value;
+        addOutputArgument(func.arguments[2]);
     }
-    else if(stringType=="TOLOWER")
+    else if(stringType=="TOLOWER" && func.arguments.count()==3)
     {
         m_type=ToLower;
         m_input.append(func.arguments[1].value);
         m_outputVariable = func.arguments[2].value;
         addOutputArgument(func.arguments[2]);
     }
-    else if(stringType=="LENGTH")
+    else if(stringType=="LENGTH" && func.arguments.count()==3)
     {
         m_type=Length;
         m_input.append(func.arguments[1].value);
         m_outputVariable = func.arguments[2].value;
-        
         addOutputArgument(func.arguments[2]);
     }
     else if(stringType=="SUBSTRING")
@@ -3098,7 +3158,7 @@ SubdirDependsAst::~SubdirDependsAst()
 
 bool SubdirDependsAst::parseFunctionInfo( const CMakeFunctionDesc& func)
 {
-    if ( func.name.toLower() != "subdir_depends" || func.arguments.isEmpty())
+    if ( func.name != "subdir_depends" || func.arguments.isEmpty())
         return false;
 
     return true;
@@ -3115,7 +3175,7 @@ SubdirsAst::~SubdirsAst()
 
 bool SubdirsAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "subdirs" )
+    if ( func.name != "subdirs" )
         return false;
 
     if ( func.arguments.isEmpty() )
@@ -3149,7 +3209,7 @@ TargetLinkLibrariesAst::~TargetLinkLibrariesAst()
 
 bool TargetLinkLibrariesAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "target_link_libraries" )
+    if ( func.name != "target_link_libraries" )
         return false;
 
     //we don't do variable expansion when parsing like CMake does, so we
@@ -3200,7 +3260,7 @@ TryCompileAst::~TryCompileAst()
 
 bool TryCompileAst::parseFunctionInfo( const CMakeFunctionDesc& func )
 {
-    if ( func.name.toLower() != "try_compile" || func.arguments.size() < 3)
+    if ( func.name != "try_compile" || func.arguments.size() < 3)
         return false;
     m_resultName=func.arguments[0].value;
     m_binDir=func.arguments[1].value;
@@ -3356,6 +3416,29 @@ bool UtilitySourceAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     return true;
 }
 
+UnsetAst::UnsetAst()
+{
+}
+
+UnsetAst::~UnsetAst()
+{
+}
+
+
+bool UnsetAst::parseFunctionInfo( const CMakeFunctionDesc& func )
+{
+    if(func.name.toLower()!="unset" || func.arguments.count()<1 || func.arguments.count()>2)
+        return false;
+    m_variableName = func.arguments.first().value;
+    addOutputArgument(func.arguments.first());
+    m_cache = func.arguments.count()==2 && func.arguments.last().value=="CACHE";
+    m_env = m_variableName.startsWith("ENV{");
+    if(m_env) {
+        m_variableName = m_variableName.mid(4, -2);
+    }
+    return func.arguments.count()==1 || (m_cache && !m_env);
+}
+
 VariableRequiresAst::VariableRequiresAst()
 {
 }
@@ -3708,6 +3791,7 @@ bool GetPropertyAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     
     QList<CMakeFunctionArgument>::const_iterator it=func.arguments.constBegin(), itEnd=func.arguments.constEnd();
     m_outputVariable=it->value;
+    addOutputArgument(*it);
     ++it;
     
     PropertyType t;
@@ -3720,6 +3804,8 @@ bool GetPropertyAst::parseFunctionInfo( const CMakeFunctionDesc& func )
     else if(propName=="VARIABLE") t=VariableProperty;
     else
         return false;
+    m_type=t;
+    
     ++it;
     if(it->value!="PROPERTY") {
         m_typeName=it->value;
