@@ -77,12 +77,13 @@ namespace CMakeParserUtils
         VariableMap m_varsDef;
         QStringList modulePathDef=QStringList(CMakeParserUtils::valueFromSystemInfo( "CMAKE_ROOT", systeminfo ) + "/Modules");
         kDebug(9042) << "found module path is" << modulePathDef;
-        m_varsDef.insert("CMAKE_BINARY_DIR", QStringList("#[bin_dir]"));
-        m_varsDef.insert("CMAKE_INSTALL_PREFIX", QStringList("#[install_dir]"));
-        m_varsDef.insert("CMAKE_COMMAND", QStringList(cmakeCmd));
-        m_varsDef.insert("CMAKE_MAJOR_VERSION", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_MAJOR_VERSION", systeminfo)));
-        m_varsDef.insert("CMAKE_MINOR_VERSION", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_MINOR_VERSION", systeminfo))); 
-        m_varsDef.insert("CMAKE_PATCH_VERSION", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_PATCH_VERSION", systeminfo)));
+        m_varsDef.insertGlobal("CMAKE_BINARY_DIR", QStringList("#[bin_dir]"));
+        m_varsDef.insertGlobal("CMAKE_INSTALL_PREFIX", QStringList("#[install_dir]"));
+        m_varsDef.insertGlobal("CMAKE_COMMAND", QStringList(cmakeCmd));
+        m_varsDef.insertGlobal("CMAKE_MAJOR_VERSION", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_MAJOR_VERSION", systeminfo)));
+        m_varsDef.insertGlobal("CMAKE_MINOR_VERSION", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_MINOR_VERSION", systeminfo))); 
+        m_varsDef.insertGlobal("CMAKE_PATCH_VERSION", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_PATCH_VERSION", systeminfo)));
+        m_varsDef.insertGlobal("CMAKE_INCLUDE_CURRENT_DIR", QStringList("OFF"));
         
         QStringList cmakeInitScripts;
         #ifdef Q_OS_WIN
@@ -98,26 +99,27 @@ namespace CMakeParserUtils
         cmakeInitScripts << "CMakeDetermineCCompiler.cmake";
         cmakeInitScripts << "CMakeDetermineCXXCompiler.cmake";
         
-        m_varsDef.insert("CMAKE_MODULE_PATH", modulePathDef);
-        m_varsDef.insert("CMAKE_ROOT", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_ROOT", systeminfo)));
+        m_varsDef.insertGlobal("CMAKE_MODULE_PATH", modulePathDef);
+        m_varsDef.insertGlobal("CMAKE_ROOT", QStringList(CMakeParserUtils::valueFromSystemInfo("CMAKE_ROOT", systeminfo)));
         
         //Defines the behaviour that can't be identified on initialization scripts
         #ifdef Q_OS_WIN32
-            m_varsDef.insert("WIN32", QStringList("1"));
-            m_varsDef.insert("CMAKE_HOST_WIN32", QStringList("1"));
+            m_varsDef.insertGlobal("WIN32", QStringList("1"));
+            m_varsDef.insertGlobal("CMAKE_HOST_WIN32", QStringList("1"));
         #else
-            m_varsDef.insert("UNIX", QStringList("1"));
-            m_varsDef.insert("CMAKE_HOST_UNIX", QStringList("1"));
+            m_varsDef.insertGlobal("UNIX", QStringList("1"));
+            m_varsDef.insertGlobal("CMAKE_HOST_UNIX", QStringList("1"));
         #endif
         #ifdef Q_OS_MAC
-            m_varsDef.insert("APPLE", QStringList("1"));
-            m_varsDef.insert("CMAKE_HOST_APPLE", QStringList("1"));
+            m_varsDef.insertGlobal("APPLE", QStringList("1"));
+            m_varsDef.insertGlobal("CMAKE_HOST_APPLE", QStringList("1"));
         #endif
         return QPair<VariableMap,QStringList>( m_varsDef, cmakeInitScripts );
     }
 
     QString executeProcess(const QString& execName, const QStringList& args)
     {
+        Q_ASSERT(!execName.isEmpty());
         kDebug(9042) << "Executing:" << execName << "::" << args /*<< "into" << *m_vars*/;
         
         KProcess p;
@@ -147,14 +149,24 @@ namespace CMakeParserUtils
         }
     }
     
-    KDevelop::ReferencedTopDUContext includeScript(const QString& file, KDevelop::ReferencedTopDUContext parent, CMakeProjectData* data, const QString& sourcedir)
+    KDevelop::ReferencedTopDUContext includeScript(const QString& file, KDevelop::ReferencedTopDUContext parent,
+                    CMakeProjectData* data, const QString& sourcedir, const QMap<QString,QString>& env)
     {
         kDebug(9042) << "Running cmake script: " << file;
         CMakeFileContent f = CMakeListsParser::readCMakeFile(file);
-        
-        data->vm.insert("CMAKE_CURRENT_BINARY_DIR", data->vm.value("CMAKE_BINARY_DIR"));
         data->vm.insert("CMAKE_CURRENT_LIST_FILE", QStringList(file));
         data->vm.insert("CMAKE_CURRENT_LIST_DIR", QStringList(QFileInfo(file).dir().absolutePath()));
+
+        const QString projectSourceDir = data->vm.value("CMAKE_SOURCE_DIR").first();
+        const QString projectBinDir = data->vm.value("CMAKE_BINARY_DIR").join(QString());
+        QString binDir = projectBinDir;
+        // CURRENT_BINARY_DIR must point to the subfolder if any"
+        if (sourcedir.startsWith(projectSourceDir)) {
+            Q_ASSERT(projectSourceDir.size()==sourcedir.size() || sourcedir.at(projectSourceDir.size()) == '/');
+            binDir += sourcedir.mid(projectSourceDir.size());
+        }
+        data->vm.insert("CMAKE_BINARY_DIR", QStringList(projectBinDir));
+        data->vm.insert("CMAKE_CURRENT_BINARY_DIR", QStringList(binDir));
         data->vm.insert("CMAKE_CURRENT_SOURCE_DIR", QStringList(sourcedir));
         
         CMakeProjectVisitor v(file, parent);
@@ -162,6 +174,7 @@ namespace CMakeParserUtils
         v.setVariableMap(&data->vm);
         v.setMacroMap(&data->mm);
         v.setModulePath(data->modulePath);
+        v.setEnvironmentProfile(env);
         v.walk(f, 0, true);
         
         data->projectName=v.projectName();
@@ -170,6 +183,7 @@ namespace CMakeParserUtils
         data->includeDirectories=v.includeDirectories();
         data->targets=v.targets();
         data->properties=v.properties();
+        data->testSuites=v.testSuites();
         
         //printSubdirectories(data->subdirectories);
         
@@ -180,7 +194,4 @@ namespace CMakeParserUtils
         
         return v.context();
     }
-
-    
 }
-    
